@@ -1,5 +1,8 @@
+from pyglossary import Glossary
+
+
 def takePhonetic_oxford_gb(glos):
-	phonGlos = Glossary() ## phonetic glossary
+	phonGlos = Glossary()  # phonetic glossary
 	phonGlos.setInfo("name", glos.getInfo("name") + "_phonetic")
 	for entry in glos:
 		word = entry.s_word
@@ -23,23 +26,24 @@ def takePhonetic_oxford_gb(glos):
 			i = defi.find(s, 2, 85)
 			if i == -1:
 				continue
-			else:
-				ph = defi[:i + 1]
-				break
-		ph = ph.replace(";", "\t")\
-			.replace(",", "\t")\
-			.replace("     ", "\t")\
-			.replace("    ", "\t")\
-			.replace("  ", "\t")\
-			.replace("//", "/")\
-			.replace("\t/\t", "\t")\
-			.replace("<i>US</i>\t", "\tUS: ")\
-			.replace("<i>US</i>", "\tUS: ")\
-			.replace("\t\t\t", "\t")\
+			ph = defi[: i + 1]
+			break
+		ph = (
+			ph.replace(";", "\t")
+			.replace(",", "\t")
+			.replace("     ", "\t")
+			.replace("    ", "\t")
+			.replace("  ", "\t")
+			.replace("//", "/")
+			.replace("\t/\t", "\t")
+			.replace("<i>US</i>\t", "\tUS: ")
+			.replace("<i>US</i>", "\tUS: ")
+			.replace("\t\t\t", "\t")
 			.replace("\t\t", "\t")
-		#	.replace("/", "")
+		)
+		# 	.replace("/", "")
 		# 	.replace("\\n ", "\\n")
 		# 	.replace("\\n ", "\\n")
-		if ph != "":
+		if ph:
 			phonGlos.addEntryObj(phonGlos.newEntry(word, ph))
 	return phonGlos
