@@ -22,23 +22,34 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from itertools import groupby
-import os
-from os.path import join
-import zipfile
-import tempfile
-from datetime import datetime
-import shutil
-
-from pyglossary.text_utils import toStr, toBytes
-from pyglossary.os_utils import indir, rmtree
+from __future__ import annotations
 
 import logging
+import os
+import shutil
+import tempfile
+import zipfile
+from datetime import datetime
+from os.path import join
+from typing import TYPE_CHECKING, cast
+
+from pyglossary.os_utils import indir, rmtree
+from pyglossary.repro_zipfile import ReproducibleZipFile as ZipFile
+
+if TYPE_CHECKING:
+	import io
+	from collections.abc import Generator
+	from typing import Any
+
+	from pyglossary.glossary_types import EntryType, WriterGlossaryType
+
+__all__ = ["EbookWriter"]
+
 log = logging.getLogger("pyglossary")
 
 
-class GroupState(object):
-	def __init__(self, writer) -> None:
+class GroupState:
+	def __init__(self, writer: EbookWriter) -> None:
 		self.writer = writer
 		self.last_prefix = ""
 		self.group_index = -1
@@ -47,14 +58,14 @@ class GroupState(object):
 	def reset(self) -> None:
 		self.first_word = ""
 		self.last_word = ""
-		self.group_contents = []
+		self.group_contents: list[str] = []
 
 	def is_new(self, prefix: str) -> bool:
-		return self.last_prefix and prefix != self.last_prefix
+		return bool(self.last_prefix) and prefix != self.last_prefix
 
-	def add(self, entry: "BaseEntry", prefix: str) -> None:
+	def add(self, entry: EntryType, prefix: str) -> None:
 		word = entry.s_word
-		defi = entry.defi
+		defi = entry.defi.replace("<br>", "<br/>").replace("<BR>", "<BR/>")
 		if not self.first_word:
 			self.first_word = word
 		self.last_word = word
@@ -62,7 +73,8 @@ class GroupState(object):
 		self.group_contents.append(self.writer.format_group_content(word, defi))
 
 
-class EbookWriter(object):
+class EbookWriter:
+
 	"""
 	A class representing a generic ebook containing a dictionary.
 
@@ -82,7 +94,7 @@ class EbookWriter(object):
 	_css: str = ""  # path to css file, or ""
 	_cover_path: str = ""  # path to cover file, or ""
 
-	CSS_CONTENTS = ""
+	CSS_CONTENTS = b""
 	GROUP_XHTML_TEMPLATE = ""
 	GROUP_XHTML_INDEX_LINK = ""
 
@@ -112,28 +124,29 @@ class EbookWriter(object):
 	</p>
 	</body>
 </html>"""
-	INDEX_XHTML_LINK_TEMPLATE = "   <span class=\"indexGroup\">" \
-		"<a href=\"{ref}\">{label}</a></span>"
+	INDEX_XHTML_LINK_TEMPLATE = (
+		'   <span class="indexGroup"><a href="{ref}">{label}</a></span>'
+	)
 
 	INDEX_XHTML_LINK_JOINER = " &#8226;\n"
 
-	OPF_MANIFEST_ITEM_TEMPLATE = "  <item href=\"{ref}\" id=\"{id}\"" \
-		" media-type=\"{mediaType}\" />"
+	OPF_MANIFEST_ITEM_TEMPLATE = (
+		'  <item href="{ref}" id="{id}" media-type="{mediaType}" />'
+	)
 
-	OPF_SPINE_ITEMREF_TEMPLATE = "  <itemref idref=\"{id}\" />"
+	OPF_SPINE_ITEMREF_TEMPLATE = '  <itemref idref="{id}" />'
 
-	def get_opf_contents(self, manifest_contents, spine_contents):
-		raise NotImplementedError
+	OPF_TEMPLATE = ""
 
 	def __init__(
 		self,
-		glos,
-		escape_strings=False,
+		glos: WriterGlossaryType,
+		escape_strings: bool = False,
 		# ignore_synonyms=False,
 		# flatten_synonyms=False,
-	):
+	) -> None:
 		self._glos = glos
-		self._filename = None
+		self._filename = ""
 
 		self._escape_strings = escape_strings
 		# self._ignore_synonyms = ignore_synonyms
@@ -145,31 +158,45 @@ class EbookWriter(object):
 		# "group_by_prefix_merge_across_first": False,
 		# "group_by_prefix_merge_min_size": 0,
 
-		self._tmpDir = None
-		self.cover = None
-		self.files = []
-		self.manifest_files = []
-		self._group_labels = []
+		self._tmpDir = tempfile.mkdtemp()
+		self.cover = ""
+		self.files: list[dict[str, Any]] = []
+		self.manifest_files: list[dict[str, str]] = []
+		self._group_labels: list[str] = []
 
-	def finish(self):
-		self._filename = None
+	def finish(self) -> None:
+		self._filename = ""
 
-	def myOpen(self, fname, mode):
-		return open(join(self._tmpDir, fname), mode)
+	def myOpen(self, fname: str, mode: str) -> io.IOBase:
+		return cast(
+			"io.IOBase",
+			open(
+				join(self._tmpDir, fname),
+				mode=mode,
+			),
+		)
 
-	def add_file(self, relative_path, contents, mode=None):
+	def add_file(
+		self,
+		relative_path: str,
+		contents: bytes,
+		mode: int | None = None,
+	) -> None:
 		if mode is None:
 			mode = zipfile.ZIP_DEFLATED
 		file_path = os.path.join(self._tmpDir, relative_path)
-		contents = toBytes(contents)
 		with self.myOpen(file_path, "wb") as file_obj:
 			file_obj.write(contents)
-		self.files.append({
-			"path": relative_path,
-			"mode": mode,
-		})
+		self.files.append(
+			{
+				"path": relative_path,
+				"mode": mode,
+			},
+		)
 
-	def write_cover(self, cover_path):
+	def write_cover(self, cover_path: str) -> None:
+		if not cover_path:
+			return
 		basename = os.path.basename(cover_path)
 		with self.myOpen(cover_path, "rb") as cover_obj:
 			cover = cover_obj.read()
@@ -182,24 +209,35 @@ class EbookWriter(object):
 		self.add_file_manifest("OEBPS/" + basename, basename, cover, mimetype)
 		self.cover = basename
 
-	def write_css(self, custom_css_path_absolute):
+	def write_css(self, custom_css_path_absolute: str) -> None:
 		css = self.CSS_CONTENTS
-		if custom_css_path_absolute is not None:
+		if custom_css_path_absolute:
 			try:
 				with self.myOpen(custom_css_path_absolute, "rb") as css_obj:
-					css = css_obj.read()
+					css = css_obj.read()  # NESTED 4
 			except Exception:
 				log.exception("")
+		if not css:
+			return
 		self.add_file_manifest("OEBPS/style.css", "style.css", css, "text/css")
 
-	def add_file_manifest(self, relative_path, id, contents, mimetype):
+	def add_file_manifest(
+		self,
+		relative_path: str,
+		id_: str,
+		contents: bytes,
+		mimetype: str,
+	) -> None:
 		self.add_file(relative_path, contents)
-		self.manifest_files.append({
-			"path": relative_path,
-			"id": id, "mimetype": mimetype,
-		})
+		self.manifest_files.append(
+			{
+				"path": relative_path,
+				"id": id_,
+				"mimetype": mimetype,
+			},
+		)
 
-	def get_group_xhtml_file_name_from_index(self, index):
+	def get_group_xhtml_file_name_from_index(self, index: int) -> str:
 		if index < self.GROUP_START_INDEX:
 			# or index >= groupCount + self.GROUP_START_INDEX:
 			# number of groups are not known, FIXME
@@ -207,49 +245,59 @@ class EbookWriter(object):
 			return "#groupPage"
 		return f"g{index:06d}.xhtml"
 
+	def _add_group(
+		self,
+		group_labels: list[str],
+		state: GroupState,
+	) -> None:
+		if not state.last_prefix:
+			return
+		state.group_index += 1
+		index = state.group_index + self.GROUP_START_INDEX
+		group_label = state.last_prefix
+		if group_label != "SPECIAL":
+			group_label = state.first_word + "&#8211;" + state.last_word
+		log.debug(f"add_group: {state.group_index}, {state.last_prefix!r}")
+		group_labels.append(group_label)
+		previous_link = self.get_group_xhtml_file_name_from_index(index - 1)
+		next_link = self.get_group_xhtml_file_name_from_index(index + 1)
+		group_xhtml_path = self.get_group_xhtml_file_name_from_index(index)
+		contents = self.GROUP_XHTML_TEMPLATE.format(
+			title=group_label,
+			group_title=group_label,
+			previous_link=previous_link,
+			index_link=(
+				self.GROUP_XHTML_INDEX_LINK if self._include_index_page else ""
+			),
+			next_link=next_link,
+			group_contents=self.GROUP_XHTML_WORD_DEFINITION_JOINER.join(
+				state.group_contents,
+			),
+		).encode("utf-8")
+		self.add_file_manifest(
+			"OEBPS/" + group_xhtml_path,
+			group_xhtml_path,
+			contents,
+			"application/xhtml+xml",
+		)
+
+	def write_data_entry(self, entry: EntryType) -> None:
+		if entry.getFileName() == "style.css":
+			self.add_file_manifest(
+				"OEBPS/style.css",
+				"style.css",
+				entry.data,
+				"text/css",
+			)
+
 	def get_prefix(self, word: str) -> str:
 		raise NotImplementedError
 
-	def sortKey(self, words: "List[str]") -> "Any":
-		raise NotImplementedError
-
-	def write_groups(self):
+	def write_groups(self) -> Generator[None, EntryType, None]:
 		# TODO: rtl=False option
 		# TODO: handle alternates better (now shows word1|word2... in title)
 
-		group_labels = []
-
-		def add_group(state):
-			if not state.last_prefix:
-				return
-			state.group_index += 1
-			index = state.group_index + self.GROUP_START_INDEX
-			group_label = state.last_prefix
-			if group_label != "SPECIAL":
-				group_label = state.first_word + "&#8211;" + state.last_word
-			log.debug(f"add_group: {state.group_index}, {state.last_prefix!r}")
-			group_labels.append(group_label)
-			previous_link = self.get_group_xhtml_file_name_from_index(index - 1)
-			next_link = self.get_group_xhtml_file_name_from_index(index + 1)
-			group_xhtml_path = self.get_group_xhtml_file_name_from_index(index)
-			self.add_file_manifest(
-				"OEBPS/" + group_xhtml_path,
-				group_xhtml_path,
-				self.GROUP_XHTML_TEMPLATE.format(
-					title=group_label,
-					group_title=group_label,
-					previous_link=previous_link,
-					index_link=(
-						self.GROUP_XHTML_INDEX_LINK
-						if self._include_index_page else ""
-					),
-					next_link=next_link,
-					group_contents=self.GROUP_XHTML_WORD_DEFINITION_JOINER.join(
-						state.group_contents,
-					),
-				),
-				"application/xhtml+xml",
-			)
+		group_labels: list[str] = []
 
 		state = GroupState(self)
 		while True:
@@ -257,60 +305,63 @@ class EbookWriter(object):
 			if entry is None:
 				break
 			if entry.isData():
-				if entry.getFileName() == "style.css":
-					self.add_file_manifest(
-						"OEBPS/style.css",
-						"style.css",
-						entry.data.decode("utf-8"),
-						"text/css",
-					)
+				self.write_data_entry(entry)
 				continue
 
 			prefix = self.get_prefix(entry.s_word)
 			if state.is_new(prefix):
-				add_group(state)
+				self._add_group(group_labels, state)
 				state.reset()
 
 			state.add(entry, prefix)
 
-		add_group(state)
+		self._add_group(group_labels, state)
 
 		self._group_labels = group_labels
 
-	def format_group_content(self, word: str, defi: str) -> str:
+	def format_group_content(
+		self,
+		word: str,
+		defi: str,
+		variants: list[str] | None = None,  # noqa: ARG002
+	) -> str:
 		return self.GROUP_XHTML_WORD_DEFINITION_TEMPLATE.format(
 			headword=self.escape_if_needed(word),
 			definition=self.escape_if_needed(defi),
 		)
 
-	def escape_if_needed(self, string):
-		if self._escape_strings:
-			string = string.replace("&", "&amp;")\
-				.replace('"', "&quot;")\
-				.replace("'", "&apos;")\
-				.replace(">", "&gt;")\
-				.replace("<", "&lt;")
-		return string
+	def escape_if_needed(self, string: str) -> str:
+		if not self._escape_strings:
+			return string
 
-	def write_index(self, group_labels):
-		"""
-			group_labels: a list of labels
-		"""
+		return (
+			string.replace("&", "&amp;")
+			.replace('"', "&quot;")
+			.replace("'", "&apos;")
+			.replace(">", "&gt;")
+			.replace("<", "&lt;")
+		)
+
+	def write_index(self, group_labels: list[str]) -> None:
+		"""group_labels: a list of labels."""
 		links = []
 		for label_i, label in enumerate(group_labels):
-			links.append(self.INDEX_XHTML_LINK_TEMPLATE.format(
-				ref=self.get_group_xhtml_file_name_from_index(
-					self.GROUP_START_INDEX + label_i
+			ref = self.get_group_xhtml_file_name_from_index(
+				self.GROUP_START_INDEX + label_i,
+			)
+			links.append(
+				self.INDEX_XHTML_LINK_TEMPLATE.format(
+					ref=ref,
+					label=label,
 				),
-				label=label,
-			))
-		links = self.INDEX_XHTML_LINK_JOINER.join(links)
+			)
+		links_str = self.INDEX_XHTML_LINK_JOINER.join(links)
 		title = self._glos.getInfo("name")
 		contents = self.INDEX_XHTML_TEMPLATE.format(
 			title=title,
 			indexTitle=title,
-			links=links,
-		)
+			links=links_str,
+		).encode("utf-8")
 		self.add_file_manifest(
 			"OEBPS/index.xhtml",
 			"index.xhtml",
@@ -318,7 +369,11 @@ class EbookWriter(object):
 			"application/xhtml+xml",
 		)
 
-	def get_opf_contents(self, manifest_contents, spine_contents):
+	def get_opf_contents(  # noqa: F811
+		self,
+		manifest_contents: str,
+		spine_contents: str,
+	) -> bytes:
 		cover = ""
 		if self.cover:
 			cover = self.COVER_TEMPLATE.format(cover=self.cover)
@@ -336,21 +391,25 @@ class EbookWriter(object):
 			cover=cover,
 			manifest=manifest_contents,
 			spine=spine_contents,
-		)
+		).encode("utf-8")
 
-	def write_opf(self):
+	def write_opf(self) -> None:
 		manifest_lines = []
 		spine_lines = []
 		for mi in self.manifest_files:
-			manifest_lines.append(self.OPF_MANIFEST_ITEM_TEMPLATE.format(
-				ref=mi["id"],
-				id=mi["id"],
-				mediaType=mi["mimetype"]
-			))
-			if mi["mimetype"] == "application/xhtml+xml":
-				spine_lines.append(self.OPF_SPINE_ITEMREF_TEMPLATE.format(
+			manifest_lines.append(
+				self.OPF_MANIFEST_ITEM_TEMPLATE.format(
+					ref=mi["id"],
 					id=mi["id"],
-				))
+					mediaType=mi["mimetype"],
+				),
+			)
+			if mi["mimetype"] == "application/xhtml+xml":
+				spine_lines.append(
+					self.OPF_SPINE_ITEMREF_TEMPLATE.format(
+						id=mi["id"],  # NESTED 4
+					),
+				)
 
 		manifest_contents = "\n".join(manifest_lines)
 		spine_contents = "\n".join(spine_lines)
@@ -361,18 +420,31 @@ class EbookWriter(object):
 
 		self.add_file("OEBPS/content.opf", opf_contents)
 
-	def write_ncx(self, group_labels):
+	def write_ncx(self, group_labels: list[str]) -> None:
 		"""
-			write_ncx
-			only for epub
-		"""
-		pass
+		write_ncx.
 
-	def open(self, filename: str):
+		only for epub.
+		"""
+
+	def open(self, filename: str) -> None:
 		self._filename = filename
-		self._tmpDir = tempfile.mkdtemp()
 
-	def write(self):
+	def _doZip(self) -> None:
+		with ZipFile(
+			self._filename,
+			mode="w",
+			compression=zipfile.ZIP_DEFLATED,
+		) as zipFp:
+			for fileDict in self.files:
+				zipFp.write(
+					fileDict["path"],
+					compress_type=fileDict["mode"],
+				)
+		if not self._keep:
+			rmtree(self._tmpDir)
+
+	def write(self) -> Generator[None, EntryType, None]:
 		filename = self._filename
 		# self._group_by_prefix_length
 		# self._include_index_page
@@ -390,45 +462,45 @@ class EbookWriter(object):
 			os.makedirs("OEBPS")
 
 			if self.MIMETYPE_CONTENTS:
-				self.add_file("mimetype", self.MIMETYPE_CONTENTS, mode=zipfile.ZIP_STORED)
+				self.add_file(
+					"mimetype",
+					self.MIMETYPE_CONTENTS.encode("utf-8"),
+					mode=zipfile.ZIP_STORED,
+				)
 			if self.CONTAINER_XML_CONTENTS:
-				self.add_file("META-INF/container.xml", self.CONTAINER_XML_CONTENTS)
+				self.add_file(
+					"META-INF/container.xml",
+					self.CONTAINER_XML_CONTENTS.encode("utf-8"),
+				)
 
-			if cover_path:
-				try:
-					self.write_cover(cover_path)
-				except Exception:
-					log.exception("")
+			try:
+				self.write_cover(cover_path)
+			except Exception:
+				log.exception("")
 
-			if css:
-				self.write_css(css)
+			self.write_css(css)
 
 			yield from self.write_groups()
 			group_labels = self._group_labels
 
 			if self._include_index_page:
-				self.write_index()
+				self.write_index(group_labels)
 
 			self.write_ncx(group_labels)
 
 			self.write_opf()
 
 			if self._compress:
-				zipFp = zipfile.ZipFile(
-					filename,
-					"w",
-					compression=zipfile.ZIP_DEFLATED,
-				)
-				for fileDict in self.files:
-					zipFp.write(
-						fileDict["path"],
-						compress_type=fileDict["mode"],
-					)
-				zipFp.close()
-				if not self._keep:
-					rmtree(self._tmpDir)
-			else:
-				if self._keep:
-					shutil.copytree(self._tmpDir, filename)
-				else:
-					shutil.move(self._tmpDir, filename)
+				self._doZip()
+				return
+
+			if self._keep:
+				shutil.copytree(self._tmpDir, filename)
+				return
+
+			if os.sep == "\\":
+				shutil.copytree(self._tmpDir, filename)
+				self._glos.addCleanupPath(self._tmpDir)
+				return
+
+			shutil.move(self._tmpDir, filename)
