@@ -18,83 +18,51 @@
 # with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
 
-import os
+from __future__ import annotations
+
+import logging
 from os.path import (
-	split,
 	splitext,
 )
-import subprocess
-import logging
 
 from .compression import (
 	stdCompressions,
 )
-from .entry import Entry
+
+__all__ = ["Error", "ReadError", "WriteError", "splitFilenameExt"]
 
 log = logging.getLogger("pyglossary")
 
+MAX_EXT_LEN = 4  # FIXME
 
-class EntryList(object):
-	def __init__(self, glos):
-		self._l = []
-		self._glos = glos
-		self._sortKey = None
 
-	def append(self, entry):
-		self._l.append(entry.getRaw(self._glos))
+class Error(Exception):
+	pass
 
-	def insert(self, pos, entry):
-		self._l.insert(pos, entry.getRaw(self._glos))
 
-	def clear(self):
-		self._l.clear()
+class ReadError(Error):
+	pass
 
-	def __len__(self):
-		return len(self._l)
 
-	def __iter__(self):
-		glos = self._glos
-		for rawEntry in self._l:
-			yield Entry.fromRaw(
-				glos, rawEntry,
-				defaultDefiFormat=glos._defaultDefiFormat,
-			)
-
-	def setSortKey(
-		self,
-		namedSortKey: "NamedSortKey",
-		sortEncoding: "Optional[str]",
-		writeOptions: "Dict[str, Any]",
-	):
-		sortKey = namedSortKey.normal(sortEncoding, **writeOptions)
-		self._sortKey = Entry.getRawEntrySortKey(self._glos, sortKey)
-
-	def sort(self):
-		if self._sortKey is None:
-			raise ValueError("EntryList.sort: sortKey is not set")
-		self._l.sort(key=self._sortKey)
-
-	def close(self):
-		pass
+class WriteError(Error):
+	pass
 
 
 def splitFilenameExt(
 	filename: str = "",
-) -> "Tuple[str, str, str]":
-	"""
-	returns (filenameNoExt, ext, compression)
-	"""
+) -> tuple[str, str, str, str]:
+	"""Return (filenameNoExt, filename, ext, compression)."""
 	compression = ""
 	filenameNoExt, ext = splitext(filename)
 	ext = ext.lower()
 
-	if not ext and len(filenameNoExt) < 5:
+	if not ext and len(filenameNoExt) <= MAX_EXT_LEN:
 		filenameNoExt, ext = "", filenameNoExt
 
 	if not ext:
 		return filename, filename, "", ""
 
-	if ext[1:] in stdCompressions + ("zip", "dz"):
+	if ext[1:] in {*stdCompressions, "zip", "dz"}:
 		compression = ext[1:]
 		filename = filenameNoExt
 		filenameNoExt, ext = splitext(filename)
