@@ -1,10 +1,14 @@
-import re
+from __future__ import annotations
+
 import base64
 import logging
+import re
 from os.path import join
 
 from pyglossary.text_utils import crc32hex
 
+__all__ = ["extractInlineHtmlImages"]
+
 log = logging.getLogger("pyglossary")
 
 re_inline_image = re.compile('src="(data:image/[^<>"]*)"')
@@ -14,21 +18,20 @@ def extractInlineHtmlImages(
 	defi: str,
 	outDir: str,
 	fnamePrefix: str = "",
-) -> "Tuple[str, List[Tuple[str, str]]]":
-	imageDataDict = {}  # type: Dict[str, bytes]
+) -> tuple[str, list[tuple[str, str]]]:
+	imageDataDict: dict[str, bytes] = {}
 
-	def subFunc(m: "Match"):
-		nonlocal images
-		src = m.group(1)[len("data:image/"):]
+	def subFunc(m: re.Match[str]) -> str:
+		src = m.group(1)[len("data:image/") :]
 		i = src.find(";")
 		if i < 0:
 			log.error(f"no semicolon, bad inline img src: {src[:60]}...")
-			return
-		imgFormat, src = src[:i], src[i + 1:]
+			return ""
+		imgFormat, src = src[:i], src[i + 1 :]
 		if not src.startswith("base64,"):
 			log.error(f"no 'base64,', bad inline img src: {src[:60]}...")
-			return
-		imgDataB64 = src[len("base64,"):]
+			return ""
+		imgDataB64 = src[len("base64,") :]
 		imgData = base64.b64decode(imgDataB64)
 		imgFname = f"{fnamePrefix}{crc32hex(imgData)}.{imgFormat}"
 		imageDataDict[imgFname] = imgData
@@ -36,7 +39,7 @@ def extractInlineHtmlImages(
 
 	defi = re_inline_image.sub(subFunc, defi)
 
-	images = []
+	images: list[tuple[str, str]] = []
 	for imgFname, imgData in imageDataDict.items():
 		imgPath = join(outDir, imgFname)
 		with open(imgPath, mode="wb") as _file:
