@@ -1,3 +1,12 @@
+__all__ = [
+	"c_author",
+	"c_name",
+	"c_publisher",
+	"c_sourceLang",
+	"c_targetLang",
+	"infoKeysAliasDict",
+]
+
 c_name = "name"
 c_sourceLang = "sourceLang"
 c_targetLang = "targetLang"
@@ -22,6 +31,6 @@ infoKeysAliasDict = {
 	##
 	# do not map "publisher" to "author"
 	##
-	# are there alternatives to "creationTime"
-	# and "lastUpdated"?
+	"date": "creationTime",
+	# are there alternatives to "lastUpdated"?
 }
