@@ -1,38 +1,40 @@
+from __future__ import annotations
+
 import unicodedata
-from collections import namedtuple
-import string
+from typing import Literal, NamedTuple
+
+__all__ = [
+	"WritingSystem",
+	"getAllWritingSystemsFromText",
+	"getWritingSystemFromText",
+	# 'unicodeNextWord',
+	"writingSystemByLowercaseName",
+	"writingSystemByName",
+	"writingSystemByUnicode",
+	"writingSystemList",
+]
 
 
-WritingSystem = namedtuple(
-	"WritingSystem", [
-		"name",
-		"iso",
-		"unicode",
-		"titleTag",
-		"direction",  # ltr | rtl | ttb
-		"comma",
-		"pop",  # population in millions
-	],
-	defaults=(
-		None,  # name
-		None,  # iso
-		[],  # unicode
-		"b",  # titleTag
-		"ltr",  # direction
-		", ",  # comma
-		0,  # pop
-	),
-)
+class WritingSystem(NamedTuple):
+	name: str
+	iso: list[tuple[int, str]] | list[tuple[int, str, str]] = []
+	unicode: list = []
+	titleTag: str = "b"
+	direction: Literal["ltr", "rtl", "ttb"] = "ltr"
+	comma: str = ", "
+	pop: int | float = 0  # population in millions
+
 
 # digits and FULLWIDTH DIGITs are considered neutral/ignored, not Latin
 
 # scripts are separated into multiple groups based on their popularity
 # (usage in multiple live languages, and number of native speakers)
 
+
 writingSystemList = [
 	WritingSystem(
 		name="Latin",
-		iso=(215, "Latn"),
+		iso=[(215, "Latn")],
 		unicode=[
 			"LATIN",
 		],
@@ -40,10 +42,9 @@ writingSystemList = [
 		comma=", ",
 		pop=4900,
 	),
-
 	WritingSystem(
 		name="Arabic",
-		iso=(160, "Arab"),
+		iso=[(160, "Arab")],
 		unicode=["ARABIC"],
 		titleTag="b",
 		direction="rtl",
@@ -52,7 +53,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Cyrillic",
-		# iso=(220, "Cyrl"),
+		iso=[(220, "Cyrl")],
 		unicode=["CYRILLIC"],
 		titleTag="b",
 		comma=", ",
@@ -60,8 +61,18 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="CJK",
-		# iso: 286=Hang, 500=Hani, 410=Hira, 412=Hrkt, 411=Kana,
-		# 501=Hans, 502=Hant
+		iso=[
+			(285, "Bopo", "BOPOMOFO"),
+			(286, "Hang", "HANGUL"),
+			(410, "Hira", "HIRAGANA"),
+			(411, "Kana", "KATAKANA"),
+			(412, "Hrkt", "KATAKANA OR HIRAGANA"),
+			(460, "Yiii", "Yi"),
+			(499, "Nshu", "NUSHU"),
+			(500, "Hani", "HAN"),  # aka Hanzi, Kanji, Hanja
+			(501, "Hans", "SIMPLIFIED HAN"),
+			(502, "Hant", "TRADITIONAL HAN"),
+		],
 		unicode=[
 			"CJK",
 			"HIRAGANA",
@@ -73,6 +84,8 @@ writingSystemList = [
 			"HALFWIDTH HANGUL",
 			"YI",  # https://en.wikipedia.org/wiki/Yi_script
 			"FULLWIDTH LATIN",
+			"BOPOMOFO",
+			"NUSHU",
 		],
 		titleTag="big",
 		comma="、",
@@ -80,18 +93,16 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Devanagari",
-		# iso=(315 , "Deva"),
+		iso=[(315, "Deva")],
 		unicode=["DEVANAGARI"],
 		titleTag="big",
 		comma=", ",
 		pop=610,
 	),
-
 	# _____________________________________________________
-
 	WritingSystem(
 		name="Armenian",
-		iso=(230, "Armn"),
+		iso=[(230, "Armn")],
 		unicode=["ARMENIAN"],
 		titleTag="big",
 		comma=", ",
@@ -99,7 +110,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Bengali-Assamese",
-		iso=(325, "Beng"),
+		iso=[(325, "Beng")],
 		unicode=["BENGALI"],
 		titleTag="big",
 		comma=", ",
@@ -107,7 +118,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Burmese",
-		iso=(350, "Mymr"),
+		iso=[(350, "Mymr")],
 		unicode=["MYANMAR"],
 		titleTag="big",
 		comma=", ",  # almost not used except in English phrases
@@ -115,7 +126,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Ge'ez",
-		iso=(430, "Ethi"),
+		iso=[(430, "Ethi")],
 		unicode=["ETHIOPIC"],
 		titleTag="big",
 		comma=", ",
@@ -123,15 +134,15 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Greek",
-		iso=(200, "Grek"),
+		iso=[(200, "Grek")],
 		unicode=["GREEK"],
-		titleTag="big",
+		titleTag="b",
 		comma=", ",
 		pop=11,
 	),
 	WritingSystem(
 		name="Gujarati",
-		iso=(320, "Gujr"),
+		iso=[(320, "Gujr")],
 		unicode=["GUJARATI"],
 		titleTag="big",
 		comma=", ",
@@ -139,7 +150,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Gurmukhi",
-		iso=(310, "Guru"),
+		iso=[(310, "Guru")],
 		unicode=["GURMUKHI"],
 		titleTag="big",
 		comma=", ",
@@ -147,7 +158,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Hebrew",
-		iso=(125, "Hebr"),
+		iso=[(125, "Hebr")],
 		unicode=["HEBREW"],
 		titleTag="big",
 		direction="rtl",
@@ -156,7 +167,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Kannada",
-		iso=(345, "Knda"),
+		iso=[(345, "Knda")],
 		unicode=["KANNADA"],
 		titleTag="big",
 		comma=", ",
@@ -164,7 +175,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Khmer",
-		iso=(355, "Khmr"),
+		iso=[(355, "Khmr")],
 		unicode=["KHMER"],
 		titleTag="big",
 		comma=", ",
@@ -172,7 +183,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Lao",
-		iso=(356, "Laoo"),
+		iso=[(356, "Laoo")],
 		unicode=["LAO"],
 		titleTag="big",
 		comma=", ",
@@ -180,7 +191,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Malayalam",
-		iso=(347, "Mlym"),
+		iso=[(347, "Mlym")],
 		unicode=["MALAYALAM"],
 		titleTag="big",
 		comma=", ",
@@ -188,7 +199,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Odia",
-		iso=(327, "Orya"),
+		iso=[(327, "Orya")],
 		unicode=["ORIYA"],
 		titleTag="big",
 		comma=", ",
@@ -196,7 +207,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Sinhala",
-		iso=(348, "Sinh"),
+		iso=[(348, "Sinh")],
 		unicode=["SINHALA"],
 		titleTag="big",
 		comma=", ",
@@ -204,15 +215,26 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Sundanese",
-		iso=(362, "Sund"),
+		iso=[(362, "Sund")],
 		unicode=["SUNDANESE"],
 		titleTag="big",
 		comma=", ",
 		pop=38,
 	),
+	WritingSystem(
+		name="Brahmi",
+		iso=[
+			(300, "Brah", "Brahmi"),
+		],
+		unicode=["BRAHMI"],
+		titleTag="big",
+		comma=", ",
+	),
 	WritingSystem(
 		name="Tamil",
-		iso=(346, "Taml"),
+		iso=[
+			(346, "Taml", "Tamil"),
+		],
 		unicode=["TAMIL"],
 		titleTag="big",
 		# Parent scripts: Brahmi, Tamil-Brahmi, Pallava
@@ -221,7 +243,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Telugu",
-		iso=(340, "Telu"),
+		iso=[(340, "Telu")],
 		unicode=["TELUGU"],
 		titleTag="big",
 		comma=", ",
@@ -229,18 +251,16 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Thai",
-		iso=(352, "Thai"),
+		iso=[(352, "Thai")],
 		unicode=["THAI"],
 		titleTag="big",
 		comma=", ",
 		pop=38,
 	),
-
 	# _____________________________________________________
-
 	WritingSystem(
 		name="Syriac",
-		iso=(135, "Syrc"),
+		iso=[(135, "Syrc")],
 		unicode=["SYRIAC"],
 		titleTag="b",
 		direction="rtl",
@@ -251,7 +271,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Tibetan",
-		iso=(330, "Tibt"),
+		iso=[(330, "Tibt")],
 		unicode=["TIBETAN"],
 		titleTag="big",
 		comma=", ",  # almost not used expect in numbers!
@@ -259,7 +279,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Georgian",
-		iso=(240, "Geor"),
+		iso=[(240, "Geor")],
 		unicode=["GEORGIAN"],
 		titleTag="big",
 		comma=", ",
@@ -267,7 +287,7 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Mongolian",
-		iso=(145, "Mong"),
+		iso=[(145, "Mong")],
 		unicode=["MONGOLIAN"],
 		titleTag="big",
 		direction="ltr",  # historically ttb?
@@ -276,99 +296,94 @@ writingSystemList = [
 	),
 	WritingSystem(
 		name="Thaana",
-		iso=(170, "Thaa"),
+		iso=[(170, "Thaa")],
 		unicode=["THAANA"],
 		titleTag="big",
 		direction="rtl",
 		comma="، ",
 		pop=0.35,
 	),
-
 	# _____________________________________________________
-
 	WritingSystem(
 		name="Javanese",
-		iso=(361, "Java"),
+		iso=[(361, "Java")],
 		unicode=["JAVANESE"],
 		titleTag="big",
 		# Since around 1945 Javanese script has largely been
 		# supplanted by Latin script to write Javanese.
 	),
 	WritingSystem(
+		# aka CANADIAN ABORIGINAL or UCAS
 		name="Canadian syllabic",
-		iso=(440, "Cans"),
+		iso=[(440, "Cans")],
 		unicode=["CANADIAN SYLLABICS"],
 		titleTag="big",
 		comma=", ",
 	),
 	WritingSystem(
 		name="Takri",
-		iso=(321, "Takr"),
+		iso=[(321, "Takr")],
 		unicode=["TAKRI"],
 		titleTag="b",
-		# comma="", FIXME
+		comma=", ",
 	),
-
 	# _____________________________________________________
-
 	WritingSystem(
 		name="SignWriting",
-		iso=(95, "Sgnw"),
+		iso=[(95, "Sgnw")],
 		unicode=["SIGNWRITING"],
 		titleTag="big",
 		direction="ttb",
 		comma="𝪇",
 	),
-
 	# _____________________________________________________
-
 	WritingSystem(
 		name="Adlam",
-		iso=(166, "Adlm"),
+		iso=[(166, "Adlm")],
 		unicode=["ADLAM"],
 		titleTag="big",
 		direction="rtl",
 	),
 	WritingSystem(
 		name="Avestan",
-		iso=(134, "Avst"),
+		iso=[(134, "Avst")],
 		unicode=["AVESTAN"],
 		titleTag="b",
 		direction="rtl",
 	),
 	WritingSystem(
 		name="Glagolitic",
-		iso=(225, "Glag"),
-		unicode=["GLAGOLITIC"],
+		iso=[(225, "Glag")],
+		unicode=["GLAGOLITIC"],  # Unicode 4.1
 		titleTag="b",
 	),
 	WritingSystem(
 		name="Khojki",
-		iso=(322, "Khoj"),
+		iso=[(322, "Khoj")],
 		unicode=["KHOJKI"],
 		titleTag="big",
 	),
 	WritingSystem(
-		name="Khudabadi",  # aka: "Khudawadi", "Sindhi"
-		iso=(318, "Sind"),
+		name="Khudabadi",  # aka: Khudawadi, "Sindhi"
+		iso=[(318, "Sind")],
 		unicode=["KHUDAWADI"],
 		titleTag="big",
 	),
 	WritingSystem(
 		name="N'Ko",
-		iso=(165, "Nkoo"),
+		iso=[(165, "Nkoo")],
 		unicode=["NKO"],
 		titleTag="big",
 	),
-
 	# _____________________________________________________
-
 	# WritingSystem(
 	# 	name="Baybayin",
-	# 	unicode=["TAGALOG"],
+	# 	iso=[(370, "Tglg")],
+	# 	unicode=["TAGALOG"],  # added in Unicode 3.2
 	# ),
 	# WritingSystem(
 	# 	name="Rejang",
+	# 	iso=[(363, "Rjng")],
 	# 	unicode=["REJANG"],
 	# ),
 	# WritingSystem(
@@ -379,23 +394,17 @@ writingSystemList = [
 	# 	name="Mwangwego",
 	# 	unicode=[],
 	# ),
-
 ]
 
-for ws in writingSystemList:
-	if not ws.name:
-		raise ValueError(f"empty name in {ws}")
+for _ws in writingSystemList:
+	if not _ws.name:
+		raise ValueError(f"empty name in {_ws}")
 
-writingSystemByUnicode = {
-	uni: ws
-	for ws in writingSystemList
-	for uni in ws.unicode
-}
+writingSystemByUnicode = {uni: ws for ws in writingSystemList for uni in ws.unicode}
 
-writingSystemByName = {
-	ws.name: ws
-	for ws in writingSystemList
-}
+writingSystemByName = {ws.name: ws for ws in writingSystemList}
+
+writingSystemByLowercaseName = {ws.name.lower(): ws for ws in writingSystemList}
 
 unicodeNextWord = {
 	"HALFWIDTH",
@@ -404,32 +413,59 @@ unicodeNextWord = {
 }
 
 
-def _getWritingSystemFromText(st: str, start: int, end: int):
-	for c in st[start:end]:
-		try:
-			unicodeWords = unicodedata.name(c).split(' ')
-		except ValueError as e:
-			# if c not in string.whitespace:
-			# 	print(f"c={c!r}, {e}")
-			continue
-		alias = unicodeWords[0]
-		ws = writingSystemByUnicode.get(alias)
+def _getWritingSystemFromChar(char: str) -> WritingSystem | None:
+	try:
+		unicodeWords = unicodedata.name(char).split(" ")
+	except ValueError:
+		# if c not in string.whitespace:
+		# 	print(f"{c=}, {e}")
+		return None
+	alias = unicodeWords[0]
+	ws = writingSystemByUnicode.get(alias)
+	if ws:
+		return ws
+	if alias in unicodeNextWord:
+		return writingSystemByUnicode.get(" ".join(unicodeWords[:2]))
+	return None
+
+
+def _getWritingSystemFromText(
+	st: str,
+	start: int,
+	end: int,
+) -> WritingSystem | None:
+	for char in st[start:end]:
+		ws = _getWritingSystemFromChar(char)
 		if ws:
 			return ws
-		if alias in unicodeNextWord:
-			ws = writingSystemByUnicode.get(" ".join(unicodeWords[:2]))
-			if ws:
-				return ws
+	return None
 
 
-def getWritingSystemFromText(st: str):
+def getWritingSystemFromText(
+	st: str,
+	beginning: bool = False,
+) -> WritingSystem | None:
 	st = st.strip()
 	if not st:
 		return None
 	# some special first words in unicodedata.name(c):
 	# "RIGHT", "ASTERISK", "MODIFIER"
-	k = (len(st) + 1) // 2 - 1
+	k = 0 if beginning else (len(st) + 1) // 2 - 1
 	ws = _getWritingSystemFromText(st, k, len(st))
 	if ws:
 		return ws
 	return _getWritingSystemFromText(st, 0, k)
+
+
+def getAllWritingSystemsFromText(
+	st: str,
+) -> set[str]:
+	st = st.strip()
+	if not st:
+		return set()
+	wsSet = set()
+	for char in st:
+		ws = _getWritingSystemFromChar(char)
+		if ws:
+			wsSet.add(ws.name)
+	return wsSet
