@@ -1,14 +1,31 @@
 # -*- coding: utf-8 -*-
+from __future__ import annotations
 
-import re
 import logging
+import re
+from typing import Any
 
+__all__ = [
+	"BoolOption",
+	"DictOption",
+	"EncodingOption",
+	"FileSizeOption",
+	"FloatOption",
+	"HtmlColorOption",
+	"IntOption",
+	"ListOption",
+	"NewlineOption",
+	"Option",
+	"StrOption",
+	"optionFromDict",
+]
 
 log = logging.getLogger("pyglossary")
 
 
-def optionFromDict(data):
+def optionFromDict(data: dict[str, Any]) -> Option:
 	className = data.pop("class")
+	optClass: type
 	if className == "Option":
 		data["typ"] = data.pop("type")
 		optClass = Option
@@ -18,19 +35,19 @@ def optionFromDict(data):
 	return optClass(**data)
 
 
-class Option(object):
-	classes = {}
+class Option:
+	classes: dict[str, type] = {}
 
 	@classmethod
-	def register(cls, optClass):
+	def register(cls: type[Option], optClass: type) -> type:
 		cls.classes[optClass.__name__] = optClass
 		return optClass
 
-	def __init__(
+	def __init__(  # noqa: PLR0913
 		self,
 		typ: str,
 		customValue: bool = False,
-		values: "Optional[List[str]]" = None,
+		values: list[Any] | None = None,
 		allowNone: bool = False,
 		comment: str = "",
 		multiline: bool = False,
@@ -54,11 +71,11 @@ class Option(object):
 		self.falseComment = falseComment
 
 	@property
-	def typeDesc(self):
+	def typeDesc(self) -> str:
 		return self.typ
 
 	@property
-	def longComment(self):
+	def longComment(self) -> str:
 		comment = self.typeDesc
 		if self.comment:
 			if comment:
@@ -66,7 +83,7 @@ class Option(object):
 			comment += self.comment
 		return comment
 
-	def toDict(self):
+	def toDict(self) -> dict[str, Any]:
 		data = {
 			"class": self.__class__.__name__,
 			"type": self.typ,
@@ -85,18 +102,19 @@ class Option(object):
 			data["falseComment"] = self.falseComment
 		return data
 
-	def evaluate(self, raw: str) -> "Tuple[Any, bool]":
-		"returns (value, isValid)"
+	@classmethod
+	def evaluate(cls, raw: str) -> tuple[Any, bool]:
+		"""Return (value, isValid)."""
 		if raw == "None":
 			return None, True
 		return raw, True
 
-	def validate(self, value):
+	def validate(self, value: Any) -> bool:  # noqa: ANN401
 		if not self.customValue:
 			if not self.values:
 				log.error(
 					f"invalid option: customValue={self.customValue!r}"
-					f", values={self.values!r}"
+					f", values={self.values!r}",
 				)
 				return False
 			return value in self.values
@@ -106,22 +124,24 @@ class Option(object):
 		return self.typ == valueType
 
 	def validateRaw(self, raw: str) -> bool:
-		"returns isValid"
+		"""Return isValid."""
 		value, isValid = self.evaluate(raw)
 		if not isValid:
 			return False
-		if not self.validate(value):
-			return False
-		return True
+		return self.validate(value)
 
-	def groupValues(self) -> "Optional[Dict[str, Any]]":
+	def groupValues(self) -> dict[str, Any] | None:  # noqa: PLR6301
 		return None
 
 
 @Option.register
 class BoolOption(Option):
-	def __init__(self, allowNone=False, **kwargs):
-		values = [False, True]
+	def __init__(
+		self,
+		allowNone: bool = False,
+		**kwargs,  # noqa: ANN003
+	) -> None:
+		values: list[bool | None] = [False, True]
 		if allowNone:
 			values.append(None)
 		Option.__init__(
@@ -130,62 +150,77 @@ class BoolOption(Option):
 			customValue=False,
 			values=values,
 			allowNone=allowNone,
-			**kwargs,
+			**kwargs,  # noqa: ANN003
 		)
 
-	def toDict(self):
+	def toDict(self) -> dict[str, Any]:
 		data = Option.toDict(self)
 		del data["customValue"]
 		del data["values"]
 		return data
 
-	def evaluate(self, raw: "Union[str, bool]") -> "Tuple[Optional[bool], bool]":
-		if raw is None or raw.lower() == "none":
+	@classmethod
+	def evaluate(
+		cls,
+		raw: str | bool,
+	) -> tuple[bool | None, bool]:
+		if raw is None:
 			return None, True
 		if isinstance(raw, bool):
 			return raw, True
-		if raw.lower() in ("yes", "true", "1"):
-			return True, True
-		if raw.lower() in ("no", "false", "0"):
-			return False, True
+		if isinstance(raw, str):
+			raw = raw.lower()
+			if raw == "none":
+				return None, True
+			if raw in {"yes", "true", "1"}:
+				return True, True
+			if raw in {"no", "false", "0"}:
+				return False, True
 		return None, False  # not valid
 
 
 @Option.register
 class StrOption(Option):
-	def __init__(self, **kwargs):
+	def __init__(
+		self,
+		**kwargs,  # noqa: ANN003
+	) -> None:
 		Option.__init__(
 			self,
 			typ="str",
-			**kwargs
+			**kwargs,
 		)
 
-	def validate(self, value):
+	def validate(self, value: Any) -> bool:  # noqa: ANN401
 		if not self.customValue:
 			if not self.values:
 				log.error(
 					f"invalid option: customValue={self.customValue!r}"
-					f", values={self.values!r}"
+					f", values={self.values!r}",
 				)
 				return False
 			return value in self.values
 		return type(value).__name__ == "str"
 
-	def groupValues(self) -> "Optional[Dict[str, Any]]":
+	def groupValues(self) -> dict[str, Any] | None:  # noqa: PLR6301
 		return None
 
 
 @Option.register
 class IntOption(Option):
-	def __init__(self, **kwargs):
+	def __init__(
+		self,
+		**kwargs,  # noqa: ANN003
+	) -> None:
 		Option.__init__(
 			self,
 			typ="int",
-			**kwargs
+			**kwargs,
 		)
 
-	def evaluate(self, raw: "Union[str, int]") -> "Tuple[Optional[int], bool]":
-		"returns (value, isValid)"
+	@classmethod
+	def evaluate(cls, raw: str | int) -> tuple[int | None, bool]:
+		"""Return (value, isValid)."""
 		try:
 			value = int(raw)
 		except ValueError:
@@ -200,29 +235,29 @@ class FileSizeOption(IntOption):
 		"kib": 1024,
 		"Ki": 1024,
 		"ki": 1024,
-
+		# ------------
 		"MiB": 1048576,
 		"mib": 1048576,
 		"Mi": 1048576,
 		"mi": 1048576,
-
+		# ------------
 		"GiB": 1073741824,
 		"gib": 1073741824,
 		"Gi": 1073741824,
 		"gi": 1073741824,
-
+		# ------------
 		"kB": 1000,
 		"kb": 1000,
 		"KB": 1000,
 		"k": 1000,
 		"K": 1000,
-
+		# ------------
 		"MB": 1000000,
 		"mb": 1000000,
 		"mB": 1000000,
 		"M": 1000000,
 		"m": 1000000,
-
+		# ------------
 		"GB": 1000000000,
 		"gb": 1000000000,
 		"gB": 1000000000,
@@ -232,19 +267,22 @@ class FileSizeOption(IntOption):
 	validPattern = "^([0-9.]+)([kKmMgG]i?[bB]?)$"
 
 	@property
-	def typeDesc(self):
+	def typeDesc(self) -> str:
 		return ""
 
-	def evaluate(self, raw: "Union[str, int]") -> "Tuple[Optional[int], bool]":
+	@classmethod
+	def evaluate(cls, raw: str | int) -> tuple[int | None, bool]:
 		if not raw:
-			return 0
+			return 0, True
 		factor = 1
-		m = re.match(self.validPattern, raw)
-		if m is not None:
-			raw, unit = m.groups()
-			factor = self.factors.get(unit)
-			if factor is None:
-				return None, False
+		if isinstance(raw, str):
+			m = re.match(cls.validPattern, raw)
+			if m is not None:
+				raw, unit = m.groups()
+				factorTmp = cls.factors.get(unit)
+				if factorTmp is None:
+					return None, False
+				factor = factorTmp
 		try:
 			value = float(raw)
 		except ValueError:
@@ -256,29 +294,35 @@ class FileSizeOption(IntOption):
 
 @Option.register
 class FloatOption(Option):
-	def __init__(self, **kwargs):
+	def __init__(
+		self,
+		**kwargs,  # noqa: ANN003
+	) -> None:
 		Option.__init__(
 			self,
 			typ="float",
-			**kwargs
+			**kwargs,
 		)
 
+	@classmethod
 	def evaluate(
-		self,
-		raw: "Union[str, float, int]",
-	) -> "Tuple[Optional[float], bool]":
-		"returns (value, isValid)"
+		cls,
+		raw: str | float,
+	) -> tuple[float | None, bool]:
+		"""Return (value, isValid)."""
 		try:
 			value = float(raw)
 		except ValueError:
 			return None, False
-		else:
-			return value, True
+		return value, True
 
 
 @Option.register
 class DictOption(Option):
-	def __init__(self, **kwargs):
+	def __init__(
+		self,
+		**kwargs,  # noqa: ANN003
+	) -> None:
 		Option.__init__(
 			self,
 			typ="dict",
@@ -288,16 +332,21 @@ class DictOption(Option):
 			**kwargs,
 		)
 
-	def toDict(self):
+	def toDict(self) -> dict[str, Any]:
 		data = Option.toDict(self)
 		del data["customValue"]
 		return data
 
-	def evaluate(self, raw: "Union[str, dict]") -> "Tuple[Optional[Dict], bool]":
+	@classmethod
+	def evaluate(
+		cls,
+		raw: str | dict,
+	) -> tuple[dict | None, bool]:
 		import ast
+
 		if isinstance(raw, dict):
 			return raw, True
-		if raw == "":
+		if raw == "":  # noqa: PLC1901
 			return None, True  # valid
 		try:
 			value = ast.literal_eval(raw)
@@ -310,24 +359,26 @@ class DictOption(Option):
 
 @Option.register
 class ListOption(Option):
-	def __init__(self, **kwargs):
+	def __init__(self, **kwargs) -> None:  # noqa: ANN003
 		Option.__init__(
 			self,
 			typ="list",
 			customValue=True,
 			allowNone=True,
 			multiline=True,
-			**kwargs,
+			**kwargs,  # noqa: ANN003
 		)
 
-	def toDict(self):
+	def toDict(self) -> dict[str, Any]:
 		data = Option.toDict(self)
 		del data["customValue"]
 		return data
 
-	def evaluate(self, raw: str) -> "Tuple[Optional[List], bool]":
+	@classmethod
+	def evaluate(cls, raw: str) -> tuple[list | None, bool]:
 		import ast
-		if raw == "":
+
+		if raw == "":  # noqa: PLC1901
 			return None, True  # valid
 		try:
 			value = ast.literal_eval(raw)
@@ -344,11 +395,11 @@ class EncodingOption(Option):
 
 	def __init__(
 		self,
-		customValue=True,
-		values=None,
-		comment=None,
-		**kwargs
-	):
+		customValue: bool = True,
+		values: list[str] | None = None,
+		comment: str | None = None,
+		**kwargs,  # noqa: ANN003
+	) -> None:
 		if values is None:
 			values = [
 				"utf-8",
@@ -386,19 +437,18 @@ class EncodingOption(Option):
 			customValue=customValue,
 			values=values,
 			comment=comment,
-			**kwargs
+			**kwargs,  # noqa: ANN003
 		)
 
-	def toDict(self):
+	def toDict(self) -> dict[str, Any]:
 		data = Option.toDict(self)
 		del data["values"]
 		return data
 
-	def groupValues(self) -> "Optional[Dict[str, Any]]":
-		from collections import OrderedDict
-		groups = OrderedDict()  # type: Dict[str, List[str]]
-		others = []  # type: List[str]
-		for value in self.values:
+	def groupValues(self) -> dict[str, Any] | None:
+		groups: dict[str, list[str]] = {}
+		others: list[str] = []
+		for value in self.values or []:
 			cats = self.re_category.findall(value)
 			if not cats:
 				others.append(value)
@@ -419,11 +469,11 @@ class EncodingOption(Option):
 class NewlineOption(Option):
 	def __init__(
 		self,
-		customValue=True,
-		values=None,
-		comment=None,
-		**kwargs
-	):
+		customValue: bool = True,
+		values: list[str] | None = None,
+		comment: str | None = None,
+		**kwargs,  # noqa: ANN003
+	) -> None:
 		if values is None:
 			values = [
 				"\r\n",
@@ -439,22 +489,47 @@ class NewlineOption(Option):
 			values=values,
 			multiline=True,
 			comment=comment,
-			**kwargs
+			**kwargs,  # noqa: ANN003
 		)
 
 
+@Option.register
+class UnicodeErrorsOption(Option):
+	def __init__(
+		self,
+		comment: str | None = None,
+	) -> None:
+		if comment is None:
+			comment = "Unicode Errors, values: `strict`, `ignore`, `replace`"
+		Option.__init__(
+			self,
+			typ="str",
+			customValue=False,
+			values=["strict", "ignore", "replace"],
+			multiline=False,
+			comment=comment,
+		)
+
+	def toDict(self) -> dict[str, Any]:
+		return {
+			"class": "UnicodeErrorsOption",
+			"type": "str",
+			"comment": self.comment,
+		}
+
+
 @Option.register
 class HtmlColorOption(Option):
-	def toDict(self):
+	def toDict(self) -> dict[str, Any]:
 		data = Option.toDict(self)
 		del data["customValue"]
 		return data
 
-	def __init__(self, **kwargs):
+	def __init__(self, **kwargs) -> None:  # noqa: ANN003
 		Option.__init__(
 			self,
 			typ="str",
 			customValue=True,
-			**kwargs
+			**kwargs,  # noqa: ANN003
 		)
 		# TODO: use a specific type?
