@@ -3,7 +3,7 @@
 # Output to Apple Dictionary xml sources for Dictionary Development Kit.
 #
 # Copyright © 2016-2019 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
-# Copyright © 2016 Ratijas <ratijas.t@me.com>
+# Copyright © 2016 ivan tkachenko me@ratijas.tk
 # Copyright © 2012-2015 Xiaoqiang Wang <xiaoqiangwang AT gmail DOT com>
 #
 # This program is a free software; you can redistribute it and/or modify
@@ -17,21 +17,27 @@
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
+from __future__ import annotations
 
 import re
+from typing import Any
 
-re_spaces = re.compile(r"[ \t\n]{2,}")
-re_title = re.compile('<[^<]+?>|"|[<>]|\xef\xbb\xbf')
-re_title_short = re.compile(r"\[.*?\]")
-re_whitespace = re.compile("(\t|\n|\r)")
+__all__ = ["title", "title_long", "title_short"]
+
+_re_spaces = re.compile(r"[ \t\n]{2,}")
+_re_title = re.compile('<[^<]+?>|"|[<>]|\xef\xbb\xbf')
+_re_title_short = re.compile(r"\[.*?\]")
+_re_whitespace = re.compile("(\t|\n|\r)")
+
+# FIXME: rename all/most functions here, add a 'fix_' prefix
 
 
-def spaces(s):
+def _spaces(s: str) -> str:
 	"""
-	strip off leading and trailing whitespaces and
+	Strip off leading and trailing whitespaces and
 	replace contiguous whitespaces with just one space.
 	"""
-	return re_spaces.sub(" ", s.strip())
+	return _re_spaces.sub(" ", s.strip())
 
 
 _brackets_sub = (
@@ -74,9 +80,9 @@ _brackets_sub = (
 )
 
 
-def brackets(s):
+def _brackets(s: str) -> str:
 	r"""
-	replace all crazy brackets with square ones [].
+	Replace all crazy brackets with square ones [].
 
 	following combinations are to replace:
 		{ \[ ... \] }
@@ -87,30 +93,28 @@ def brackets(s):
 	if "{" in s:
 		for exp, sub in _brackets_sub:
 			s = exp.sub(sub, s)
-	return spaces(s)
+	return _spaces(s)
 
 
-def truncate(text, length=449):
+def _truncate(text: str, length: int = 449) -> str:
 	"""
-	trunct a string to given length
+	Trunct a string to given length
 	:param str text:
 	:return: truncated text
-	:rtype: str
+	:rtype: str.
 	"""
-	content = re_whitespace.sub(" ", text)
+	content = _re_whitespace.sub(" ", text)
 	if len(text) > length:
 		# find the next space after max_len chars (do not break inside a word)
 		pos = content[:length].rfind(" ")
 		if pos == -1:
 			pos = length
 		text = text[:pos]
-	return text
+	return text  # noqa: RET504
 
 
-def title(title, BeautifulSoup):
-	"""
-	strip double quotes and html tags.
-	"""
+def title(title: str, BeautifulSoup: Any) -> str:
+	"""Strip double quotes and html tags."""
 	if BeautifulSoup:
 		title = title.replace("\xef\xbb\xbf", "")
 		if len(title) > 1:
@@ -121,22 +125,32 @@ def title(title, BeautifulSoup):
 				# FIXME: html or lxml? gives warning unless it's lxml
 			).get_text(strip=True)
 	else:
-		title = re_title.sub("", title)
+		title = _re_title.sub("", title)
 		title = title.replace("&", "&amp;")
-	title = brackets(title)
-	title = truncate(title, 1126)
-	return title
+	title = _brackets(title)
+	title = _truncate(title, 1126)
+	return title  # noqa: RET504
 
 
-def title_long(s):
+def title_long(s: str) -> str:
 	"""
-	title_long("str[ing]") -> "string"
+	Return long title line.
+
+	Example:
+	-------
+	title_long("str[ing]") -> string.
+
 	"""
 	return s.replace("[", "").replace("]", "")
 
 
-def title_short(s):
+def title_short(s: str) -> str:
 	"""
-	title_short("str[ing]") -> "str"
+	Return short title line.
+
+	Example:
+	-------
+	title_short("str[ing]") -> str.
+
 	"""
-	return spaces(re_title_short.sub("", s))
+	return _spaces(_re_title_short.sub("", s))
