@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # appledict/indexes/ru.py
 #
-# Copyright © 2016 Ratijas <ratijas.t@me.com>
+# Copyright © 2016 ivan tkachenko me@ratijas.tk
 #
 # This program is a free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,41 +14,46 @@
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
-"""
-Russian indexes based on pymorphy.
-"""
+"""Russian indexes based on pymorphy."""
+
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+	from collections.abc import Sequence
+
+from pyglossary.core import log, pip
 
 from . import languages
-from pyglossary.plugins.formats_common import log, pip
 
 try:
-	import pymorphy2
+	import pymorphy3  # type: ignore
 except ImportError:
-	log.error(f"""module pymorphy2 is required to build extended Russian indexes.
-You can download it here: http://pymorphy2.readthedocs.org/en/latest/.
-Or by running: {pip} install pymorphy2""")
+	log.error(
+		f"""module pymorphy3 is required to build extended Russian indexes.
+You can download it here: https://github.com/no-plagiarism/pymorphy3
+Or by running: {pip} install pymorphy3""",
+	)
 	raise
 
-morphy = pymorphy2.MorphAnalyzer()
+morphy = pymorphy3.MorphAnalyzer()
 
 
-def ru(titles, _):
+def ru(titles: Sequence[str], _: str) -> set[str]:
 	"""
-	gives a set of all declines, cases and other froms of word `title`.
+	Give a set of all declines, cases and other forms of word `title`.
 	note that it works only if title is one word.
-
-	:type titles: Sequence[str]
-	:rtype: Set[str]
 	"""
-	indexes = set()
-	indexes_norm = set()
+	indexes: set[str] = set()
+	indexes_norm: set[str] = set()
 	for title in titles:
 		# in-place modification
 		_ru(title, indexes, indexes_norm)
-	return list(sorted(indexes))
+	return indexes
 
 
-def _ru(title, a, a_norm):
+def _ru(title: str, a: set[str], a_norm: set[str]) -> None:
 	# uppercase abbreviature
 	if title.isupper():
 		return
@@ -62,11 +67,9 @@ def _ru(title, a, a_norm):
 
 	# decline only one-word titles
 	if len(title.split()) == 1:
-
 		normal_forms = morphy.parse(title)
 
 		if len(normal_forms) > 0:
-
 			# forms of most probable match
 			normal_form = normal_forms[0]
 
@@ -76,14 +79,14 @@ def _ru(title, a, a_norm):
 				# "й" and "и", "ё" and "е", so we're trying to avoid
 				# "* Duplicate index. Skipped..." warning.
 				# new: return indexes with original letters but check for
-				# occurence against "normal forms".
+				# occurrence against "normal forms".
 				word_norm = normalize(word)
 				if word_norm not in a_norm:
 					a.add(word)
 					a_norm.add(word_norm)
 
 
-def normalize(word):
+def normalize(word: str) -> str:
 	return word.lower().replace("й", "и").replace("ё", "е").replace("-", " ")
 
 
