@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# mypy: ignore-errors
 #
 # Copyright © 2008-2021 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 #
@@ -16,13 +17,26 @@
 # with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
 
-from pyglossary.plugins.formats_common import *
-from .bgl_reader import BglReader as Reader
-from .bgl_reader import optionsProp
+from .reader import Reader, optionsProp
+
+__all__ = [
+	"Reader",
+	"description",
+	"enable",
+	"extensionCreate",
+	"extensions",
+	"kind",
+	"lname",
+	"name",
+	"optionsProp",
+	"singleFile",
+	"website",
+	"wiki",
+]
 
 enable = True
 lname = "babylon_bgl"
-format = "BabylonBgl"
+name = "BabylonBgl"
 description = "Babylon (.BGL)"
 extensions = (".bgl",)
 extensionCreate = ""
