@@ -1,26 +1,56 @@
-from .glossary_type import GlossaryType
-from .entry import Entry, BaseEntry
 
-import re
-from operator import itemgetter
+from __future__ import annotations
 
 import logging
+import re
+import typing
+from collections.abc import Iterable, Iterator
+from operator import itemgetter
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+	from .glossary_types import EntryType
+
+__all__ = ["reverseGlossary"]
+
 log = logging.getLogger("pyglossary")
 
+if TYPE_CHECKING:
+	class _GlossaryType(typing.Protocol):
+		def __iter__(self) -> Iterator[EntryType]: ...
+
+		def getInfo(self, key: str) -> str: ...
+
+		def progressInit(
+			self,
+			*args,  # noqa: ANN002
+		) -> None: ...
+
+		def progress(self, pos: int, total: int, unit: str = "entries") -> None: ...
+
+		def progressEnd(self) -> None: ...
+
+		@property
+		def progressbar(self) -> bool: ...
+
+		@progressbar.setter
+		def progressbar(self, enabled: bool) -> None: ...
+
+
 
 def reverseGlossary(
-	glos: GlossaryType,
+	glos: _GlossaryType,
 	savePath: str = "",
-	words: "Optional[List[str]]" = None,
+	words: list[str] | None = None,
 	includeDefs: bool = False,
 	reportStep: int = 300,
 	saveStep: int = 1000,  # set this to zero to disable auto saving
-	**kwargs
-) -> "Iterator[int]":
+	**kwargs,
+) -> Iterator[int]:
 	"""
 	This is a generator
 	Usage:
-		for wordIndex in glos.reverse(...):
+		for wordIndex in reverseGlossary(glos, ...):
 			pass
 
 	Inside the `for` loop, you can pause by waiting (for input or a flag)
@@ -38,7 +68,7 @@ def reverseGlossary(
 		minWordLen = 3
 		includeDefs = False
 		showRel = "None"
-			allowed values: "None", "Percent", "Percent At First"
+			allowed values: None, "Percent", "Percent At First"
 	"""
 	if not savePath:
 		savePath = glos.getInfo("name") + ".txt"
@@ -46,11 +76,7 @@ def reverseGlossary(
 	if saveStep < 2:
 		raise ValueError("saveStep must be more than 1")
 
-	ui = glos.ui
-
-	entries = []
-	for entry in glos:
-		entries.append(entry)
+	entries: list[EntryType] = list(glos)
 	log.info(f"loaded {len(entries)} entries into memory")
 
 	if words:
@@ -61,12 +87,12 @@ def reverseGlossary(
 	wordCount = len(words)
 	log.info(
 		f"Reversing to file {savePath!r}"
-		f", number of words: {wordCount}"
+		f", number of words: {wordCount}",
 	)
 	glos.progressInit("Reversing")
 	wcThreshold = wordCount // 200 + 1
 
-	with open(savePath, "w") as saveFile:
+	with open(savePath, "w", encoding="utf-8") as saveFile:
 		for wordI in range(wordCount):
 			word = words[wordI]
 			if wordI % wcThreshold == 0:
@@ -78,7 +104,7 @@ def reverseGlossary(
 				entries,
 				word,
 				includeDefs=includeDefs,
-				**kwargs
+				**kwargs,
 			)
 			if result:
 				try:
@@ -88,7 +114,7 @@ def reverseGlossary(
 						defi = ", ".join(result) + "."
 				except Exception:
 					log.exception("")
-					log.debug(f"result = {result}")
+					log.debug(f"{result = }")
 					return
 				saveFile.write(f"{word}\t{defi}\n")
 			yield wordI
@@ -98,24 +124,24 @@ def reverseGlossary(
 
 
 def takeOutputWords(
-	glos: GlossaryType,
-	entryIter: "Iterator[BaseEntry]",
+	glos: _GlossaryType,
+	entryIter: Iterable[EntryType],
 	minWordLen: int = 3,
-) -> "List[str]":
+) -> list[str]:
 	# fr"[\w]{{{minWordLen},}}"
-	wordPattern = re.compile(r"[\w]{%d,}" % minWordLen, re.U)
+	wordPattern = re.compile(r"[\w]{%d,}" % minWordLen, re.UNICODE)
 	words = set()
-	progressbar, glos._progressbar = glos._progressbar, False
+	progressbar, glos.progressbar = glos.progressbar, False
 	for entry in entryIter:
 		words.update(wordPattern.findall(
 			entry.defi,
 		))
-	glos._progressbar = progressbar
+	glos.progressbar = progressbar
 	return sorted(words)
 
 
 def searchWordInDef(
-	entryIter: "Iterator[BaseEntry]",
+	entryIter: Iterable[EntryType],
 	st: str,
 	matchWord: bool = True,
 	sepChars: str = ".,،",
@@ -124,21 +150,21 @@ def searchWordInDef(
 	minWordLen: int = 3,
 	includeDefs: bool = False,
 	showRel: str = "Percent",  # "Percent" | "Percent At First" | ""
-) -> "List[str]":
+) -> list[str]:
 	# searches word "st" in definitions of the glossary
 	splitPattern = re.compile(
-		"|".join([re.escape(x) for x in sepChars]),
-		re.U,
+		"|".join(re.escape(x) for x in sepChars),
+		re.UNICODE,
 	)
-	wordPattern = re.compile(r"[\w]{%d,}" % minWordLen, re.U)
-	outRel = []
+	wordPattern = re.compile(r"[\w]{%d,}" % minWordLen, re.UNICODE)
+	outRel: list[tuple[str, float] | tuple[str, float, str]] = []
 	for entry in entryIter:
 		words = entry.l_word
 		defi = entry.defi
 		if st not in defi:
 			continue
 		for word in words:
-			rel = 0  # relation value of word (0 <= rel <= 1)
+			rel = 0.0  # relation value of word (0 <= rel <= 1)
 			for part in splitPattern.split(defi):
 				if not part:
 					continue
@@ -150,12 +176,12 @@ def searchWordInDef(
 						continue
 					rel = max(
 						rel,
-						partWords.count(st) / len(partWords)
+						partWords.count(st) / len(partWords),
 					)
 				else:
 					rel = max(
 						rel,
-						part.count(st) * len(st) / len(part)
+						part.count(st) * len(st) / len(part),
 					)
 			if rel <= minRel:
 				continue
@@ -176,7 +202,7 @@ def searchWordInDef(
 	if includeDefs:
 		for j in range(n):
 			numP = num
-			w, num, m = outRel[j]
+			w, num, m = outRel[j]  # type: ignore
 			m = m.replace("\n", "\\n").replace("\t", "\\t")
 			onePer = int(1.0 / num)
 			if onePer == 1.0:
@@ -193,7 +219,7 @@ def searchWordInDef(
 		return out
 	for j in range(n):
 		numP = num
-		w, num = outRel[j]
+		w, num = outRel[j]  # type: ignore
 		onePer = int(1.0 / num)
 		if onePer == 1.0:
 			out.append(w)
