@@ -15,158 +15,136 @@
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
+from __future__ import annotations
 
-import string
-import sys
-import os
-import re
-import struct
 import binascii
 import logging
+import re
+import struct
+import sys
+from typing import AnyStr
 
-from . import core
+__all__ = [
+	"crc32hex",
+	"escapeNTB",
+	"excMessage",
+	"fixUtf8",
+	"joinByBar",
+	"replacePostSpaceChar",
+	"splitByBar",
+	"splitByBarUnescapeNTB",
+	"toStr",
+	"uint32FromBytes",
+	"uint32ToBytes",
+	"uint64FromBytes",
+	"uint64ToBytes",
+	"uintFromBytes",
+	"unescapeBar",
+	"unescapeNTB",
+	"urlToPath",
+]
 
 log = logging.getLogger("pyglossary")
 
 endFormat = "\x1b[0;0;0m"  # len=8
 
 
-def toBytes(s: "AnyStr") -> bytes:
-	return bytes(s, "utf-8") if isinstance(s, str) else bytes(s)
+def toStr(s: AnyStr) -> str:
+	if isinstance(s, bytes):
+		return str(s, "utf-8")
+	return str(s)
 
 
-def toStr(s: "AnyStr") -> str:
-	return str(s, "utf-8") if isinstance(s, bytes) else str(s)
-
-
-def fixUtf8(st: "AnyStr") -> str:
-	return toBytes(st).replace(b"\x00", b"").decode("utf-8", "replace")
+def fixUtf8(st: AnyStr) -> str:
+	if isinstance(st, str):
+		return st.encode("utf-8").replace(b"\x00", b"").decode("utf-8", "replace")
+	return st.replace(b"\x00", b"").decode("utf-8", "replace")
 
 
 pattern_n_us = re.compile(r"((?<!\\)(?:\\\\)*)\\n")
 pattern_t_us = re.compile(r"((?<!\\)(?:\\\\)*)\\t")
 pattern_bar_us = re.compile(r"((?<!\\)(?:\\\\)*)\\\|")
 pattern_bar_sp = re.compile(r"(?:(?<!\\)(?:\\\\)*)\|")
-b_pattern_bar_us = re.compile(r"((?<!\\)(?:\\\\)*)\\\|".encode("ascii"))
-
-
-def replaceStringTable(
-	rplList: "List[Tuple[str, str]]",
-) -> "Callable[[str], str]":
-	def replace(st: str) -> str:
-		for rpl in rplList:
-			st = st.replace(rpl[0], rpl[1])
-		return st
-	return replace
 
 
 def escapeNTB(st: str, bar: bool = False) -> str:
-	"""
-		scapes Newline, Tab, Baskslash, and vertical Bar (if bar=True)
-	"""
+	"""Scapes Newline, Tab, Baskslash, and vertical Bar (if bar=True)."""
 	st = st.replace("\\", "\\\\")
 	st = st.replace("\t", r"\t")
 	st = st.replace("\r", "")
 	st = st.replace("\n", r"\n")
 	if bar:
 		st = st.replace("|", r"\|")
-	return st
+	return st  # noqa: RET504
 
 
 def unescapeNTB(st: str, bar: bool = False) -> str:
-	"""
-		unscapes Newline, Tab, Baskslash, and vertical Bar (if bar=True)
-	"""
+	"""Unscapes Newline, Tab, Baskslash, and vertical Bar (if bar=True)."""
 	st = pattern_n_us.sub("\\1\n", st)
 	st = pattern_t_us.sub("\\1\t", st)
 	if bar:
 		st = pattern_bar_us.sub(r"\1|", st)
 	st = st.replace("\\\\", "\\")  # probably faster than re.sub
-	return st
+	return st  # noqa: RET504
 
 
-def splitByBarUnescapeNTB(st: str) -> "List[str]":
+def splitByBarUnescapeNTB(st: str) -> list[str]:
+	r"""
+	Split by "|" (and not "\\|") then unescapes Newline (\\n),
+	Tab (\\t), Baskslash (\\) and Bar (\\|) in each part
+	returns a list.
 	"""
-		splits by "|" (and not "\\|") then unescapes Newline (\\n),
-			Tab (\\t), Baskslash (\\) and Bar (\\|) in each part
-		returns a list
-	"""
-	return [
-		unescapeNTB(part, bar=True)
-		for part in pattern_bar_sp.split(st)
-	]
+	return [unescapeNTB(part, bar=True) for part in pattern_bar_sp.split(st)]
 
 
 def escapeBar(st: str) -> str:
-	r"""
-		scapes vertical bar (\|)
-	"""
-	st = st.replace("\\", "\\\\")
-	st = st.replace("|", r"\|")
-	return st
+	r"""Scapes vertical bar (\|)."""
+	return st.replace("\\", "\\\\").replace("|", r"\|")
 
 
 def unescapeBar(st: str) -> str:
+	r"""Unscapes vertical bar (\|)."""
+	# str.replace is probably faster than re.sub
+	return pattern_bar_us.sub(r"\1|", st).replace("\\\\", "\\")
+
+
+def splitByBar(st: str) -> list[str]:
 	r"""
-		unscapes vertical bar (\|)
+	Split by "|" (and not "\\|")
+	then unescapes Baskslash (\\) and Bar (\\|) in each part.
 	"""
-	st = pattern_bar_us.sub(r"\1|", st)
-	st = st.replace("\\\\", "\\")  # probably faster than re.sub
-	return st
+	return [unescapeBar(part) for part in pattern_bar_sp.split(st)]
 
 
-def splitByBar(st: str) -> "List[str]":
-	"""
-		splits by "|" (and not "\\|")
-		then unescapes Baskslash (\\) and Bar (\\|) in each part
-	"""
-	return [
-		unescapeBar(part)
-		for part in pattern_bar_sp.split(st)
-	]
-
-
-def joinByBar(parts: "List[str]") -> "str":
-	return "|".join([
-		escapeBar(part)
-		for part in parts
-	])
-
-
-def unescapeBarBytes(st: bytes) -> bytes:
-	r"""
-		unscapes vertical bar (\|)
-	"""
-	st = b_pattern_bar_us.sub(b"\\1|", st)
-	st = st.replace(b"\\\\", b"\\")  # probably faster than re.sub
-	return st
+def joinByBar(parts: list[str]) -> str:
+	return "|".join(escapeBar(part) for part in parts)
 
 
 # return a message string describing the current exception
 def excMessage() -> str:
 	i = sys.exc_info()
+	if not i[0]:
+		return ""
 	return f"{i[0].__name__}: {i[1]}"
 
 
-def formatHMS(h: int, m: int, s: int) -> str:
-	if h == 0:
-		if m == 0:
-			return f"{s:02d}"
-		else:
-			return f"{m:02d}:{s:02d}"
-	else:
-		return f"{h:02d}:{m:02d}:{s:02d}"
-
-
 # ___________________________________________ #
 
 
 def uint32ToBytes(n: int) -> bytes:
-	return struct.pack('>I', n)
+	return struct.pack(">I", n)
+
+
+def uint64ToBytes(n: int) -> bytes:
+	return struct.pack(">Q", n)
 
 
 def uint32FromBytes(bs: bytes) -> int:
-	return struct.unpack('>I', bs)[0]
+	return struct.unpack(">I", bs)[0]
+
+
+def uint64FromBytes(bs: bytes) -> int:
+	return struct.unpack(">Q", bs)[0]
 
 
 def uintFromBytes(bs: bytes) -> int:
@@ -177,13 +155,15 @@ def uintFromBytes(bs: bytes) -> int:
 
 
 def crc32hex(bs: bytes) -> str:
-	return struct.pack('>I', binascii.crc32(bs) & 0xffffffff).hex()
+	return struct.pack(">I", binascii.crc32(bs) & 0xFFFFFFFF).hex()
+
 
 # ___________________________________________ #
 
 
 def urlToPath(url: str) -> str:
 	from urllib.parse import unquote
+
 	if not url.startswith("file://"):
 		return unquote(url)
 	path = url[7:]
@@ -196,18 +176,9 @@ def urlToPath(url: str) -> str:
 
 
 def replacePostSpaceChar(st: str, ch: str) -> str:
-	st = (
+	return (
 		st.replace(f" {ch}", ch)
 		.replace(ch, f"{ch} ")
 		.replace(f"{ch}  ", f"{ch} ")
+		.removesuffix(" ")
 	)
-	if st.endswith(" "):
-		st = st[:-1]
-	return st
-
-
-def isASCII(data: str) -> bool:
-	for c in data:
-		if ord(c) >= 128:
-			return False
-	return True
