@@ -1,22 +1,34 @@
-import os
+from __future__ import annotations
+
 import logging
+import os
 from os.path import (
 	isdir,
-	splitext,
 )
-from pyglossary.compression import compressionOpen as c_open
+from typing import TYPE_CHECKING, cast
+
+if TYPE_CHECKING:
+	import io
+	from collections.abc import Callable, Generator
+
+	from .glossary_types import EntryType, WriterGlossaryType
+
+from .compression import compressionOpen as c_open
+from .io_utils import nullTextIO
+
+__all__ = ["TextGlossaryWriter", "writeTxt"]
 
 log = logging.getLogger("pyglossary")
 
 file_size_check_every = 100
 
 
-class TextGlossaryWriter(object):
-	_encoding = "utf-8"
-	_newline = "\n"
-	_wordListEncodeFunc: "Optional[Callable[[List[str]], str]]" = None
-	_wordEscapeFunc: "Optional[Callable[[str], str]]" = None
-	_defiEscapeFunc: "Optional[Callable[[str], str]]" = None
+class TextGlossaryWriter:
+	_encoding: str = "utf-8"
+	_newline: str = "\n"
+	_wordListEncodeFunc: Callable[[list[str]], str] | None = None
+	_wordEscapeFunc: Callable[[str], str] | None = None
+	_defiEscapeFunc: Callable[[str], str] | None = None
 	_ext: str = ".txt"
 	_head: str = ""
 	_tail: str = ""
@@ -26,14 +38,14 @@ class TextGlossaryWriter(object):
 
 	def __init__(
 		self,
-		glos: "GlossaryType",
+		glos: WriterGlossaryType,
 		entryFmt: str = "",  # contain {word} and {defi}
 		writeInfo: bool = True,
-		outInfoKeysAliasDict: "Optional[Dict[str, str]]" = None,
+		outInfoKeysAliasDict: dict[str, str] | None = None,
 	) -> None:
 		self._glos = glos
 		self._filename = ""
-		self._file = None
+		self._file: io.TextIOBase = nullTextIO
 		self._resDir = ""
 
 		if not entryFmt:
@@ -41,26 +53,24 @@ class TextGlossaryWriter(object):
 
 		self._entryFmt = entryFmt
 		self._writeInfo = writeInfo
-
-		if not outInfoKeysAliasDict:
-			outInfoKeysAliasDict = {}
-		self._outInfoKeysAliasDict = outInfoKeysAliasDict
+		self._outInfoKeysAliasDict = outInfoKeysAliasDict or {}
 		# TODO: replace outInfoKeysAliasDict arg with a func?
 
-	def setAttrs(
+	# TODO: use @property setters
+	def setAttrs(  # noqa: PLR0913
 		self,
-		encoding=None,
-		newline=None,
-		wordListEncodeFunc=None,
-		wordEscapeFunc=None,
-		defiEscapeFunc=None,
-		ext=None,
-		head=None,
-		tail=None,
-		resources=None,
-		word_title=None,
-		file_size_approx=None,
-	):
+		encoding: str | None = None,
+		newline: str | None = None,
+		wordListEncodeFunc: Callable | None = None,
+		wordEscapeFunc: Callable | None = None,
+		defiEscapeFunc: Callable | None = None,
+		ext: str | None = None,
+		head: str | None = None,
+		tail: str | None = None,
+		resources: bool | None = None,
+		word_title: bool | None = None,
+		file_size_approx: int | None = None,
+	) -> None:
 		if encoding is not None:
 			self._encoding = encoding
 		if newline is not None:
@@ -89,53 +99,64 @@ class TextGlossaryWriter(object):
 			self._glos.setInfo("file_count", "-1")
 		self._open(filename)
 		self._filename = filename
-		self._resDir = f"{filename}_res"
-		if not isdir(self._resDir):
-			os.mkdir(self._resDir)
+		if not self._glos.getConfig("skip_resources", False):
+			self._resDir = f"{filename}_res"
+			if not isdir(self._resDir):
+				os.mkdir(self._resDir)
 
-	def _open(self, filename: str):
+	def _doWriteInfo(self, file: io.TextIOBase) -> None:
+		entryFmt = self._entryFmt
+		outInfoKeysAliasDict = self._outInfoKeysAliasDict
+		wordEscapeFunc = self._wordEscapeFunc
+		defiEscapeFunc = self._defiEscapeFunc
+		for key, value in self._glos.iterInfo():
+			# both key and value are supposed to be non-empty string
+			if not (key and value):
+				log.warning(f"skipping info {key=}, {value=}")
+				continue
+			key = outInfoKeysAliasDict.get(key, key)  # noqa: PLW2901
+			if not key:
+				continue
+			word = f"##{key}"
+			if wordEscapeFunc is not None:
+				word = wordEscapeFunc(word)
+				if not word:
+					continue
+			if defiEscapeFunc is not None:
+				value = defiEscapeFunc(value)  # noqa: PLW2901
+				if not value:
+					continue
+			file.write(
+				entryFmt.format(
+					word=word,
+					defi=value,
+				),
+			)
+
+	def _open(self, filename: str) -> io.TextIOBase:
 		if not filename:
 			filename = self._glos.filename + self._ext
 
-		_file = self._file = c_open(
-			filename,
-			mode="wt",
-			encoding=self._encoding,
-			newline=self._newline,
+		file = self._file = cast(
+			"io.TextIOBase",
+			c_open(
+				filename,
+				mode="wt",
+				encoding=self._encoding,
+				newline=self._newline,
+			),
 		)
-		_file.write(self._head)
-		if self._writeInfo:
-			entryFmt = self._entryFmt
-			outInfoKeysAliasDict = self._outInfoKeysAliasDict
-			wordEscapeFunc = self._wordEscapeFunc
-			defiEscapeFunc = self._defiEscapeFunc
-			for key, value in self._glos.iterInfo():
-				# both key and value are supposed to be non-empty string
-				if not (key and value):
-					log.warning(f"skipping info key={key!r}, value={value!r}")
-					continue
-				key = outInfoKeysAliasDict.get(key, key)
-				if not key:
-					continue
-				word = f"##{key}"
-				if wordEscapeFunc is not None:
-					word = wordEscapeFunc(word)
-					if not word:
-						continue
-				if defiEscapeFunc is not None:
-					value = defiEscapeFunc(value)
-					if not value:
-						continue
-				_file.write(entryFmt.format(
-					word=word,
-					defi=value,
-				))
-		_file.flush()
-		return _file
+		file.write(self._head)
 
-	def write(self):
+		if self._writeInfo:
+			self._doWriteInfo(file)
+
+		file.flush()
+		return file
+
+	def write(self) -> Generator[None, EntryType, None]:
 		glos = self._glos
-		_file = self._file
+		file = self._file
 		entryFmt = self._entryFmt
 		wordListEncodeFunc = self._wordListEncodeFunc
 		wordEscapeFunc = self._wordEscapeFunc
@@ -146,6 +167,7 @@ class TextGlossaryWriter(object):
 		file_size_approx = self._file_size_approx
 		entryCount = 0
 		fileIndex = 0
+		glosName = self._glos.getInfo("name") or self._filename
 
 		while True:
 			entry = yield
@@ -171,16 +193,20 @@ class TextGlossaryWriter(object):
 
 			if defiEscapeFunc is not None:
 				defi = defiEscapeFunc(defi)
-			_file.write(entryFmt.format(word=word, defi=defi))
+			file.write(entryFmt.format(word=word, defi=defi))
 
 			if file_size_approx > 0:
 				entryCount += 1
-				if entryCount % file_size_check_every == 0:
-					if _file.tell() >= file_size_approx:
-						fileIndex += 1
-						_file = self._open(f"{self._filename}.{fileIndex}")
+				if (
+					entryCount % file_size_check_every == 0
+					and file.tell() >= file_size_approx
+				):
+					fileIndex += 1
+					log.info(f"Creating {self._filename}.{fileIndex}")
+					self._glos.setInfo("name", f"{glosName} part {fileIndex + 1}")
+					file = self._open(f"{self._filename}.{fileIndex}")
 
-	def finish(self):
+	def finish(self) -> None:
 		if self._tail:
 			self._file.write(self._tail)
 		self._file.close()
@@ -188,22 +214,22 @@ class TextGlossaryWriter(object):
 			os.rmdir(self._resDir)
 
 
-def writeTxt(
-	glos: "GlossaryType",
+def writeTxt(  # noqa: PLR0913
+	glos: WriterGlossaryType,
 	entryFmt: str = "",  # contain {word} and {defi}
 	filename: str = "",
 	writeInfo: bool = True,
-	wordEscapeFunc: "Optional[Callable]" = None,
-	defiEscapeFunc: "Optional[Callable]" = None,
+	wordEscapeFunc: Callable | None = None,
+	defiEscapeFunc: Callable | None = None,
 	ext: str = ".txt",
 	head: str = "",
 	tail: str = "",
-	outInfoKeysAliasDict: "Optional[Dict[str, str]]" = None,
+	outInfoKeysAliasDict: dict[str, str] | None = None,
 	encoding: str = "utf-8",
 	newline: str = "\n",
 	resources: bool = True,
 	word_title: bool = False,
-) -> "Generator[None, BaseEntry, None]":
+) -> Generator[None, EntryType, None]:
 	writer = TextGlossaryWriter(
 		glos,
 		entryFmt=entryFmt,
@@ -224,27 +250,3 @@ def writeTxt(
 	writer.open(filename)
 	yield from writer.write()
 	writer.finish()
-
-
-def writeTabfile(
-	glos: "GlossaryType",
-	filename: str = "",
-	encoding: str = "utf-8",
-	resources: bool = True,
-) -> "Generator[None, BaseEntry, None]":
-	from pyglossary.text_utils import escapeNTB
-	writer = TextGlossaryWriter(
-		glos,
-		entryFmt="{word}\t{defi}\n",
-		outInfoKeysAliasDict=None,
-	)
-	writer.setAttrs(
-		encoding=encoding,
-		wordEscapeFunc=escapeNTB,
-		defiEscapeFunc=escapeNTB,
-		ext=".txt",
-		resources=resources,
-	)
-	writer.open(filename)
-	yield from writer.write()
-	writer.finish()
