@@ -18,21 +18,23 @@
 # with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
 
-from pyglossary.glossary import Glossary
-
 # reqs = subprocess.check_output([sys.executable, '-m', 'pip', 'freeze'])
 # ^ this takes about 3 seconds
 # installed_packages = set(r.decode().split('==')[0] for r in reqs.split())
 
+from __future__ import annotations
 
-def checkDepends(depends: "Dict[str, str]") -> "List[str]":
-	"returns the list of uninstalled dependencies"
+__all__ = ["checkDepends"]
+
+
+def checkDepends(depends: dict[str, str]) -> list[str]:
+	"""Return the list of non-installed dependencies."""
 	if not depends:
 		return []
-	uninstalled = []
+	not_installed = []
 	for moduleName, pkgName in depends.items():
 		try:
 			__import__(moduleName)
-		except ModuleNotFoundError:
-			uninstalled.append(pkgName)
-	return uninstalled
+		except ModuleNotFoundError:  # noqa: PERF203
+			not_installed.append(pkgName)
+	return not_installed
