@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# mypy: ignore-errors
 #
 # Copyright © 2016-2017 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 #
@@ -15,23 +16,28 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 
-from gi.repository import Gtk as gtk
+
+from collections.abc import Callable
+
 from gi.repository import Gdk as gdk
+from gi.repository import Gtk as gtk
+
+__all__ = ["MyDialog"]
 
 
-class MyDialog(object):
-	def startWaiting(self):
+class MyDialog:
+	def startWaiting(self) -> None:
 		self.queue_draw()
 		self.vbox.set_sensitive(False)
 		self.get_window().set_cursor(gdk.Cursor.new(gdk.CursorType.WATCH))
 		while gtk.events_pending():
 			gtk.main_iteration_do(False)
 
-	def endWaiting(self):
+	def endWaiting(self) -> None:
 		self.get_window().set_cursor(gdk.Cursor.new(gdk.CursorType.LEFT_PTR))
 		self.vbox.set_sensitive(True)
 
-	def waitingDo(self, func, *args, **kwargs):
+	def waitingDo(self, func: Callable, *args, **kwargs) -> None:  # noqa: ANN002
 		self.startWaiting()
 		try:
 			func(*args, **kwargs)
