@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# mypy: ignore-errors
 #
 # Copyright © 2016-2017 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 #
@@ -15,21 +16,32 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 
-from . import *
-from .utils import *
+
+from __future__ import annotations
+
+from typing import Any
+
+from . import gdk, gtk
+from .utils import imageFromFile
+
+__all__ = ["ResizeButton"]
 
 
 class ResizeButton(gtk.EventBox):
-	def __init__(self, win, edge=gdk.WindowEdge.SOUTH_EAST):
+	def __init__(
+		self,
+		win: gtk.Window,
+		edge: gdk.WindowEdge = gdk.WindowEdge.SOUTH_EAST,
+	) -> None:
 		gtk.EventBox.__init__(self)
 		self.win = win
 		self.edge = edge
 		###
-		self.image = imageFromFile('resize.png')
+		self.image = imageFromFile("resize.png")
 		self.add(self.image)
-		self.connect('button-press-event', self.buttonPress)
+		self.connect("button-press-event", self.buttonPress)
 
-	def buttonPress(self, obj, gevent):
+	def buttonPress(self, _obj: Any, gevent: gdk.ButtonEvent) -> None:
 		self.win.begin_resize_drag(
 			self.edge,
 			gevent.button,
