@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# mypy: ignore-errors
 # ui_cmd.py
 #
 # Copyright © 2008-2021 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
@@ -17,17 +18,27 @@
 # You should have received a copy of the GNU General Public License along
 # with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
+from __future__ import annotations
 
+import os
+import sys
 from os.path import join
-import time
+from typing import TYPE_CHECKING, Any
 
-from pyglossary.glossary import *
-from .base import *
-from . import progressbar as pb
+from pyglossary.core import dataDir, log
+from pyglossary.glossary_v2 import ConvertArgs, Error, Glossary
+
+from .base import UIBase, fread
 from .wcwidth import wcswidth
 
+if TYPE_CHECKING:
+	import logging
+	from collections.abc import Mapping
 
-def wc_ljust(text, length, padding=' '):
+__all__ = ["COMMAND", "UI", "parseFormatOptionsStr", "printHelp"]
+
+
+def wc_ljust(text: str, length: int, padding: str = " ") -> str:
 	return text + padding * max(0, (length - wcswidth(text)))
 
 
@@ -45,62 +56,60 @@ else:
 COMMAND = "pyglossary"
 
 
-def getColWidth(subject, strings):
-	return max(
-		len(x) for x in [subject] + strings
-	)
+def getColWidth(subject: str, strings: list[str]) -> int:
+	return max(len(x) for x in [subject] + strings)
 
 
-def getFormatsTable(names, header):
-	descriptions = [
-		Glossary.plugins[name].description
-		for name in names
-	]
-	extensions = [
-		" ".join(Glossary.plugins[name].extensions)
-		for name in names
-	]
+def getFormatsTable(names: list[str], header: str) -> str:
+	descriptions = [Glossary.plugins[name].description for name in names]
+	extensions = [" ".join(Glossary.plugins[name].extensions) for name in names]
 
 	nameWidth = getColWidth("Name", names)
 	descriptionWidth = getColWidth("Description", descriptions)
 	extensionsWidth = getColWidth("Extensions", extensions)
 
-	lines = ["\n"]
-	lines.append(startBold + header + endFormat)
-
-	lines.append(
-		" | ".join([
-			"Name".center(nameWidth),
-			"Description".center(descriptionWidth),
-			"Extensions".center(extensionsWidth)
-		])
-	)
-	lines.append(
-		"-+-".join([
-			"-" * nameWidth,
-			"-" * descriptionWidth,
-			"-" * extensionsWidth,
-		])
-	)
+	lines = [
+		"\n",
+		startBold + header + endFormat,
+		" | ".join(
+			[
+				"Name".center(nameWidth),
+				"Description".center(descriptionWidth),
+				"Extensions".center(extensionsWidth),
+			],
+		),
+		"-+-".join(
+			[
+				"-" * nameWidth,
+				"-" * descriptionWidth,
+				"-" * extensionsWidth,
+			],
+		),
+	]
 	for index, name in enumerate(names):
 		lines.append(
-			" | ".join([
-				name.ljust(nameWidth),
-				descriptions[index].ljust(descriptionWidth),
-				extensions[index].ljust(extensionsWidth)
-			])
+			" | ".join(
+				[
+					name.ljust(nameWidth),
+					descriptions[index].ljust(descriptionWidth),
+					extensions[index].ljust(extensionsWidth),
+				],
+			),
 		)
 
 	return "\n".join(lines)
 
 
-def help():
+def printHelp() -> None:
 	import string
+
 	text = fread(join(dataDir, "help"))
-	text = text.replace("<b>", startBold)\
-		.replace("<u>", startUnderline)\
-		.replace("</b>", endFormat)\
+	text = (
+		text.replace("<b>", startBold)
+		.replace("<u>", startUnderline)
+		.replace("</b>", endFormat)
 		.replace("</u>", endFormat)
+	)
 	text = string.Template(text).substitute(
 		CMD=COMMAND,
 	)
@@ -109,16 +118,14 @@ def help():
 	print(text)
 
 
-def parseFormatOptionsStr(st) -> "Optional[Dict]":
-	"""
-		prints error and returns None if failed to parse one option
-	"""
-
+# TODO: raise exception instead of returning None
+def parseFormatOptionsStr(st: str) -> dict[str, str] | None:
+	"""Prints error and returns None if failed to parse one option."""
 	st = st.strip()
 	if not st:
 		return {}
 
-	opt = {}
+	opt: dict[str, str] = {}
 	parts = st.split(";")
 	for part in parts:
 		if not part:
@@ -131,43 +138,48 @@ def parseFormatOptionsStr(st) -> "Optional[Dict]":
 		if not key:
 			log.critical(f"bad option syntax: {part!r}")
 			return None
-		value = part[eq + 1:].strip()
+		value = part[eq + 1 :].strip()
 		opt[key] = value
 	return opt
 
 
-def encodeFormatOptions(opt: "Dict") -> str:
-	if not opt:
-		return ""
-	parts = []
-	for key, value in opt.items():
-		parts.append(f"{key}={value}")
-	return ";".join(parts)
-
-
-class NullObj(object):
-	def __getattr__(self, attr):
+class NullObj:
+	def __getattr__(self, attr: str) -> NullObj:
 		return self
 
-	def __setattr__(self, attr, value):
+	def __setattr__(self, attr: str, value: Any) -> None:
 		pass
 
-	def __setitem__(self, key, value):
+	def __setitem__(self, key: str, value: Any) -> None:
 		pass
 
-	def __call__(self, *args, **kwargs):
+	def __call__(
+		self,
+		*args: tuple[Any],
+		**kwargs: Mapping[str, Any],
+	) -> None:
 		pass
 
+	def __bool__(self) -> bool:
+		return False
+
 
 class UI(UIBase):
-	def __init__(self):
+	def __init__(
+		self,
+		progressbar: bool = True,
+	) -> None:
 		UIBase.__init__(self)
 		# log.debug(self.config)
 		self.pbar = NullObj()
 		self._toPause = False
 		self._resetLogFormatter = None
+		self._progressbar = progressbar
 
-	def onSigInt(self, *args):
+	def onSigInt(
+		self,
+		*_args: tuple[Any],
+	) -> None:
 		log.info("")
 		if self._toPause:
 			log.info("Operation Canceled")
@@ -176,54 +188,56 @@ class UI(UIBase):
 			self._toPause = True
 			log.info("Please wait...")
 
-	def setText(self, text):
+	def setText(self, text: str) -> None:
 		self.pbar.widgets[0] = text
 
-	def fixLogger(self):
+	def fixLogger(self) -> None:
 		for h in log.handlers:
 			if h.name == "std":
 				self.fixLogHandler(h)
 				return
 
-	def fillMessage(self, msg):
-		return wc_ljust(msg, self.pbar.term_width)
+	def fillMessage(self, msg: str) -> str:
+		term_width = self.pbar.term_width
+		if not term_width:
+			return msg
+		return "\r" + wc_ljust(msg, term_width)
 
-	def fixLogHandler(self, h):
-		def reset():
+	def fixLogHandler(self, h: logging.Handler) -> None:
+		def reset() -> None:
 			h.formatter.fill = None
 
 		self._resetLogFormatter = reset
 		h.formatter.fill = self.fillMessage
 
-	def progressInit(self, title):
-		rot = pb.RotatingMarker()
-		self.pbar = pb.ProgressBar(
-			maxval=1.0,
-			# update_step=0.5, removed
-		)
-		self.pbar.widgets = [
-			title + " ",
-			pb.AnimatedMarker(),
-			" ",
-			pb.Bar(marker="█"),
-			pb.Percentage(), " ",
-			pb.ETA(),
-		]
-		self.pbar.start(num_intervals=1000)
-		rot.pbar = self.pbar
+	def progressInit(self, title: str) -> None:
+		try:
+			from .pbar_tqdm import createProgressBar
+		except ModuleNotFoundError:
+			from .pbar_legacy import createProgressBar
+		self.pbar = createProgressBar(title)
 		self.fixLogger()
 
-	def progress(self, rat, text=""):
-		self.pbar.update(rat)
+	def progress(
+		self,
+		ratio: float,
+		text: str = "",  # noqa: ARG002
+	) -> None:
+		self.pbar.update(ratio)
 
-	def progressEnd(self):
+	def progressEnd(self) -> None:
 		self.pbar.finish()
 		if self._resetLogFormatter:
 			self._resetLogFormatter()
 
-	def reverseLoop(self, *args, **kwargs):
+	def reverseLoop(
+		self,
+		*_args: tuple[Any],
+		**kwargs: Mapping[Any],
+	) -> None:
 		from pyglossary.reverse import reverseGlossary
-		reverseKwArgs = {}
+
+		reverseKwArgs: dict[str, Any] = {}
 		for key in (
 			"words",
 			"matchWord",
@@ -233,7 +247,7 @@ class UI(UIBase):
 			"saveStep",
 			"maxNum",
 			"minRel",
-			"minWordLen"
+			"minWordLen",
 		):
 			try:
 				reverseKwArgs[key] = self.config["reverse_" + key]
@@ -243,28 +257,28 @@ class UI(UIBase):
 
 		if not self._toPause:
 			log.info("Reversing glossary... (Press Ctrl+C to pause/stop)")
-		for wordI in reverseGlossary(self.glos, **reverseKwArgs):
+		for _ in reverseGlossary(self.glos, **reverseKwArgs):
 			if self._toPause:
 				log.info(
-					"Reverse is paused."
-					" Press Enter to continue, and Ctrl+C to exit"
+					"Reverse is paused. Press Enter to continue, and Ctrl+C to exit",
 				)
 				input()
 				self._toPause = False
 
-	def run(
+	# PLR0912 Too many branches (19 > 12)
+	def run(  # noqa: PLR0912, PLR0913
 		self,
 		inputFilename: str = "",
 		outputFilename: str = "",
 		inputFormat: str = "",
 		outputFormat: str = "",
 		reverse: bool = False,
-		config: "Optional[Dict]" = None,
-		readOptions: "Optional[Dict]" = None,
-		writeOptions: "Optional[Dict]" = None,
-		convertOptions: "Optional[Dict]" = None,
-		glossarySetAttrs: "Optional[Dict]" = None,
-	):
+		config: dict[str, Any] | None = None,
+		readOptions: dict[str, Any] | None = None,
+		writeOptions: dict[str, Any] | None = None,
+		convertOptions: dict[str, Any] | None = None,
+		glossarySetAttrs: dict[str, Any] | None = None,
+	) -> bool:
 		if config is None:
 			config = {}
 		if readOptions is None:
@@ -278,16 +292,16 @@ class UI(UIBase):
 
 		self.config = config
 
-		if inputFormat:
+		if inputFormat:  # noqa: SIM102
 			# inputFormat = inputFormat.capitalize()
 			if inputFormat not in Glossary.readFormats:
 				log.error(f"invalid read format {inputFormat}")
-		if outputFormat:
+		if outputFormat:  # noqa: SIM102
 			# outputFormat = outputFormat.capitalize()
 			if outputFormat not in Glossary.writeFormats:
 				log.error(f"invalid write format {outputFormat}")
 				log.error(f"try: {COMMAND} --help")
-				return 1
+				return False
 		if not outputFilename:
 			if reverse:
 				pass
@@ -297,44 +311,51 @@ class UI(UIBase):
 				except (KeyError, IndexError):
 					log.error(f"invalid write format {outputFormat}")
 					log.error(f"try: {COMMAND} --help")
-					return 1
-				else:
-					outputFilename = os.path.splitext(inputFilename)[0] + ext
+					return False
+				outputFilename = os.path.splitext(inputFilename)[0] + ext
 			else:
 				log.error("neither output file nor output format is given")
 				log.error(f"try: {COMMAND} --help")
-				return 1
+				return False
 
 		glos = self.glos = Glossary(ui=self)
-		self.glos.config = self.config
+		glos.config = self.config
+		glos.progressbar = self._progressbar
 
 		for attr, value in glossarySetAttrs.items():
 			setattr(glos, attr, value)
 
 		if reverse:
 			import signal
-			signal.signal(signal.SIGINT, self.onSigInt)  # good place? FIXME
+
+			signal.signal(signal.SIGINT, self.onSigInt)
 			readOptions["direct"] = True
 			if not glos.read(
 				inputFilename,
-				format=inputFormat,
-				**readOptions
+				formatName=inputFormat,
+				**readOptions,
 			):
 				log.error("reading input file was failed!")
 				return False
 			self.setText("Reversing: ")
 			self.pbar.update_step = 0.1
 			self.reverseLoop(savePath=outputFilename)
-		else:
-			finalOutputFile = self.glos.convert(
-				inputFilename,
-				inputFormat=inputFormat,
-				outputFilename=outputFilename,
-				outputFormat=outputFormat,
-				readOptions=readOptions,
-				writeOptions=writeOptions,
-				**convertOptions
-			)
-			return bool(finalOutputFile)
+			return True
 
-		return True
+		try:
+			finalOutputFile = self.glos.convert(
+				ConvertArgs(
+					inputFilename,
+					inputFormat=inputFormat,
+					outputFilename=outputFilename,
+					outputFormat=outputFormat,
+					readOptions=readOptions,
+					writeOptions=writeOptions,
+					**convertOptions,
+				),
+			)
+		except Error as e:
+			log.critical(str(e))
+			glos.cleanup()
+			return False
+		return bool(finalOutputFile)
