@@ -1,5 +1,5 @@
-#!/usr/bin/python3
 # -*- coding: utf-8 -*-
+# mypy: ignore-errors
 # ui_cmd_interactive.py
 #
 # Copyright © 2008-2022 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
@@ -19,16 +19,18 @@
 # with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
 
+from __future__ import annotations
+
 """
 To use this user interface:
-	sudo pip3 install prompt_toolkit
+sudo pip3 install prompt_toolkit.
 """
 
 # GitHub repo for prompt_toolkit
 # https://github.com/prompt-toolkit/python-prompt-toolkit
 
 # The code for Python's cmd.Cmd was very ugly and hard to understand last I
-# cheched. But we don't use cmd module here, and nor does prompt_toolkit.
+# checked. But we don't use cmd module here, and nor does prompt_toolkit.
 
 # Completion func for Python's readline, silently (and stupidly) hides any
 # exception, and only shows the print if it's in the first line of function.
@@ -40,101 +42,68 @@ To use this user interface:
 # prompt_toolkit also supports ncurses-like dialogs with buttons and widgets,
 # but I prefer this kind of UI with auto-completion and history
 
-import sys
-import os
-from os.path import (
-	dirname,
-	join,
-	abspath,
-	relpath,
-	isdir,
-	isabs,
-)
-import logging
-from collections import OrderedDict
 import argparse
-import shlex
-
 import json
+import logging
+import os
+import shlex
+from os.path import (
+	abspath,
+	dirname,
+	isabs,
+	isdir,
+	join,
+	relpath,
+)
+from typing import TYPE_CHECKING, Any
+
+if TYPE_CHECKING:
+	from collections.abc import Iterable
+
+	from prompt_toolkit.completion import CompleteEvent
+	from prompt_toolkit.document import Document
+	from prompt_toolkit.formatted_text import StyleAndTextTuples
+	from prompt_toolkit.key_binding.key_processor import KeyPressEvent
+
+	from pyglossary.option import Option
+	from pyglossary.plugin_prop import PluginProp
+
+from prompt_toolkit import ANSI
+from prompt_toolkit import prompt as promptLow
+from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
+from prompt_toolkit.completion import (
+	Completion,
+	PathCompleter,
+	WordCompleter,
+)
+from prompt_toolkit.history import FileHistory
+from prompt_toolkit.key_binding import KeyBindings
+from prompt_toolkit.keys import Keys
+from prompt_toolkit.shortcuts import PromptSession, confirm
 
 from pyglossary import core
 from pyglossary.core import confDir
-from pyglossary.glossary import Glossary
+from pyglossary.glossary_v2 import Error, Glossary
+from pyglossary.sort_keys import lookupSortKey, namedSortKeyList
 from pyglossary.ui import ui_cmd
-from pyglossary.sort_keys import namedSortKeyList, namedSortKeyByName
-
-
-from prompt_toolkit import prompt as promptLow
-from prompt_toolkit.history import FileHistory
-from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
-from prompt_toolkit.completion import (
-	WordCompleter,
-	PathCompleter,
-	Completion,
-)
-from prompt_toolkit import ANSI
-from prompt_toolkit.shortcuts import confirm, PromptSession
-from prompt_toolkit.key_binding import KeyBindings
-from prompt_toolkit.keys import Keys
+from pyglossary.ui.termcolors import colors
 
+__all__ = ["UI"]
 
 endFormat = "\x1b[0;0;0m"
 
 
-class MiniCheckBoxPrompt(object):
+class MiniCheckBoxPrompt:
 	def __init__(
 		self,
-		message: str = "",
-		fmt: str = "{message}: {check}",
+		formatted: StyleAndTextTuples,
 		value: bool = False,
-	):
-		self.message = message
-		self.fmt = fmt
+	) -> None:
+		self.formatted = formatted
 		self.value = value
 
-	def formatMessage(self):
-		msg = self.fmt.format(
-			check="[x]" if self.value else "[ ]",
-			message=self.message,
-		)
-		# msg = ANSI(msg)  # NOT SUPPORTED
-		return msg
-
-	def __pt_formatted_text__(self):
-		return [("", self.formatMessage())]
-
-
-def checkbox_prompt(
-	message: str,
-	default: bool,
-) -> PromptSession[bool]:
-	"""
-	Create a `PromptSession` object for the 'confirm' function.
-	"""
-	bindings = KeyBindings()
-
-	check = MiniCheckBoxPrompt(message=message, value=default)
-
-	@bindings.add(" ")
-	def space(event: "E") -> None:
-		check.value = not check.value
-		# cursor_pos = check.formatMessage().find("[") + 1
-		# cur_cursor_pos = session.default_buffer.cursor_position
-		# print(f"cur_cursor_pos={cur_cursor_pos}, cursor_pos={cursor_pos}")
-		# session.default_buffer.cursor_position = cursor_pos
-
-	@bindings.add(Keys.Any)
-	def _(event: "E") -> None:
-		" Disallow inserting other text. "
-		pass
-
-	complete_message = check
-	session: PromptSession[bool] = PromptSession(
-		complete_message,
-		key_bindings=bindings
-	)
-	session.prompt()
-	return check.value
+	def __pt_formatted_text__(self) -> StyleAndTextTuples:  # noqa: PLW3201
+		return self.formatted + [("", "[x]" if self.value else "[ ]")]
 
 
 log = logging.getLogger("pyglossary")
@@ -150,23 +119,17 @@ for direc in (cmdiConfDir, histDir):
 if __name__ == "__main__":
 	Glossary.init()
 
-pluginByDesc = {
-	plugin.description: plugin
-	for plugin in Glossary.plugins.values()
-}
+pluginByDesc = {plugin.description: plugin for plugin in Glossary.plugins.values()}
 readFormatDescList = [
-	Glossary.plugins[_format].description
-	for _format in Glossary.readFormats
+	Glossary.plugins[_format].description for _format in Glossary.readFormats
 ]
 writeFormatDescList = [
-	Glossary.plugins[_format].description
-	for _format in Glossary.writeFormats
+	Glossary.plugins[_format].description for _format in Glossary.writeFormats
 ]
 
 convertOptionsFlags = {
 	"direct": ("indirect", "direct"),
 	"sqlite": ("", "sqlite"),
-	"progressbar": ("no-progress-bar", ""),
 	"sort": ("no-sort", "sort"),
 }
 infoOverrideFlags = {
@@ -176,7 +139,11 @@ infoOverrideFlags = {
 }
 
 
-def dataToPrettyJson(data, ensure_ascii=False, sort_keys=False):
+def dataToPrettyJson(
+	data: dict[str, Any] | list[Any],
+	ensure_ascii: bool = False,
+	sort_keys: bool = False,
+) -> str:
 	return json.dumps(
 		data,
 		sort_keys=sort_keys,
@@ -186,10 +153,10 @@ def dataToPrettyJson(data, ensure_ascii=False, sort_keys=False):
 
 
 def prompt(
-	message: str,
+	message: ANSI | str,
 	multiline: bool = False,
 	**kwargs,
-):
+) -> str:
 	if kwargs.get("default", "") is None:
 		kwargs["default"] = ""
 	text = promptLow(message=message, **kwargs)
@@ -198,9 +165,9 @@ def prompt(
 		text = promptLow(
 			message="",
 			multiline=True,
-			**kwargs
+			**kwargs,
 		)
-	return text
+	return text  # noqa: RET504
 
 
 back = "back"
@@ -209,31 +176,30 @@ back = "back"
 class MyPathCompleter(PathCompleter):
 	def __init__(
 		self,
-		reading: bool,
-		fs_action_names=None,
-		**kwargs
-	):
+		reading: bool,  # noqa: ARG002
+		fs_action_names: list[str] | None = None,
+		**kwargs,
+	) -> None:
 		PathCompleter.__init__(
 			self,
 			file_filter=self.file_filter,
-			**kwargs
+			**kwargs,
 		)
-		if fs_action_names is None:
-			fs_action_names = []
-		self.fs_action_names = fs_action_names
+		self.fs_action_names = fs_action_names or []
 
-	def file_filter(self, filename: str) -> bool:
-		# filename is full/absoule file path
+	@staticmethod
+	def file_filter(_filename: str) -> bool:
+		# filename is full/absolute file path
 		return True
 
 	# def get_completions_exception(document, complete_event, e):
-	# 	log.error(f"Execption in get_completions: {e}")
+	# 	log.error(f"Exception in get_completions: {e}")
 
 	def get_completions(
 		self,
-		document: "Document",
-		complete_event: "CompleteEvent",
-	) -> "Iterable[Completion]":
+		document: Document,
+		complete_event: CompleteEvent,
+	) -> Iterable[Completion]:
 		text = document.text_before_cursor
 
 		for action in self.fs_action_names:
@@ -252,29 +218,32 @@ class MyPathCompleter(PathCompleter):
 
 
 class AbsolutePathHistory(FileHistory):
-	def load_history_strings(self) -> "Iterable[str]":
+	def load_history_strings(self) -> Iterable[str]:
 		# pwd = os.getcwd()
 		pathList = FileHistory.load_history_strings(self)
-		return [
-			relpath(p)
-			for p in pathList
-		]
+		return [relpath(p) for p in pathList]
 
 	def store_string(self, string: str) -> None:
 		FileHistory.store_string(self, abspath(string))
 
 
 class UI(ui_cmd.UI):
-	def __init__(self):
+	def __init__(
+		self,
+		progressbar: bool = True,
+	) -> None:
 		self._inputFilename = ""
 		self._outputFilename = ""
 		self._inputFormat = ""
 		self._outputFormat = ""
-		self.config = None
-		self._readOptions = None
-		self._writeOptions = None
-		self._convertOptions = None
-		ui_cmd.UI.__init__(self)
+		self.config: dict[str, Any] = {}
+		self._readOptions = {}
+		self._writeOptions = {}
+		self._convertOptions = {}
+		ui_cmd.UI.__init__(
+			self,
+			progressbar=progressbar,
+		)
 
 		self.ls_parser = argparse.ArgumentParser(add_help=False)
 		self.ls_parser.add_argument(
@@ -291,48 +260,52 @@ class UI(ui_cmd.UI):
 			help="display help",
 		)
 		self.ls_usage = (
-			'Usage: !ls [--help] [-l] [FILE/DIRECTORY]...\n\n'
-			'optional arguments:\n'
-			'    --help      show this help message and exit\n'
-			'    -l, --long  use a long listing format\n'
+			"Usage: !ls [--help] [-l] [FILE/DIRECTORY]...\n\n"
+			"optional arguments:\n"
+			"    --help      show this help message and exit\n"
+			"    -l, --long  use a long listing format\n"
 		)
 
-		self._fsActions = OrderedDict([
-			("!pwd", (self.fs_pwd, "")),
-			("!ls", (self.fs_ls, self.ls_usage)),
-			("!..", (self.fs_cd_parent, "")),
-			("!cd", (self.fs_cd, "")),
-		])
-		self._finalActions = OrderedDict([
-			("formats", self.askFormats),
-			("read-options", self.askReadOptions),
-			("write-options", self.askWriteOptions),
-			("reset-read-options", self.resetReadOptions),
-			("reset-write-options", self.resetWriteOptions),
-			("config", self.askConfig),
-			("indirect", self.setIndirect),
-			("sqlite", self.setSQLite),
-			("no-progressbar", self.setNoProgressbar),
-			("sort", self.setSort),
-			("sort-key", self.setSortKey),
-			("show-options", self.showOptions),
-			("back", None),
-		])
+		self._fsActions = {
+			"!pwd": (self.fs_pwd, ""),
+			"!ls": (self.fs_ls, self.ls_usage),
+			"!..": (self.fs_cd_parent, ""),
+			"!cd": (self.fs_cd, ""),
+		}
+		self._finalActions = {
+			"formats": self.askFormats,
+			"read-options": self.askReadOptions,
+			"write-options": self.askWriteOptions,
+			"reset-read-options": self.resetReadOptions,
+			"reset-write-options": self.resetWriteOptions,
+			"config": self.askConfig,
+			"indirect": self.setIndirect,
+			"sqlite": self.setSQLite,
+			"no-progressbar": self.setNoProgressbar,
+			"sort": self.setSort,
+			"sort-key": self.setSortKey,
+			"show-options": self.showOptions,
+			"back": None,
+		}
 
-	def fs_pwd(self, args: "List[str]"):
+	@staticmethod
+	def fs_pwd(args: list[str]) -> None:
+		if args:
+			print(f"extra arguments: {args}")
 		print(os.getcwd())
 
+	@staticmethod
 	def get_ls_l(
-		self,
 		arg: str,
-		st: "Optional[os.stat_result]" = None,
+		st: os.stat_result | None = None,
 		parentDir: str = "",
 		sizeWidth: int = 0,
 	) -> str:
-		import stat
-		import pwd
 		import grp
+		import pwd
+		import stat
 		import time
+
 		argPath = arg
 		if parentDir:
 			argPath = join(parentDir, arg)
@@ -351,7 +324,7 @@ class UI(ui_cmd.UI):
 			details.append(f"-> {os.readlink(argPath)}")
 		return "  ".join(details)
 
-	def fs_ls(self, args: "List[str]"):
+	def fs_ls(self, args: list[str]) -> None:
 		opts, args = self.ls_parser.parse_known_args(args=args)
 
 		if opts.help:
@@ -364,8 +337,8 @@ class UI(ui_cmd.UI):
 		showTitle = len(args) > 1
 		# Note: isdir and isfile funcs follow sym links, so no worry about links
 
-		for i, arg in enumerate(args):
-			if i > 0:
+		for argI, arg in enumerate(args):
+			if argI > 0:
 				print()
 
 			if not isdir(arg):
@@ -376,31 +349,29 @@ class UI(ui_cmd.UI):
 				print(f"> List of directory {arg!r}:")
 
 			if not opts.long:
-				for _path in os.listdir(arg):
-					if isdir(_path):
-						_path += "/"
-					print(f"{_path}")
+				for path in os.listdir(arg):
+					if isdir(path):
+						print(f"{path}/")
+					else:
+						print(f"{path}")
 				continue
 
 			contents = os.listdir(arg)
-			statList = [
-				os.lstat(join(arg, _path))
-				for _path in contents
-			]
-			maxFileSize = max([
-				st.st_size
-				for st in statList
-			])
+			statList = [os.lstat(join(arg, _path)) for _path in contents]
+			maxFileSize = max(st.st_size for st in statList)
 			sizeWidth = len(str(maxFileSize))
-			for i, _path in enumerate(contents):
-				print(self.get_ls_l(
-					_path,
-					parentDir=arg,
-					st=statList[i],
-					sizeWidth=sizeWidth,
-				))
+			for pathI, path_ in enumerate(contents):
+				print(
+					self.get_ls_l(
+						path_,
+						parentDir=arg,
+						st=statList[pathI],
+						sizeWidth=sizeWidth,
+					),
+				)
 
-	def fs_cd_parent(self, args: "List[str]"):
+	@staticmethod
+	def fs_cd_parent(args: list[str]) -> None:
 		if args:
 			log.error("This command does not take arguments")
 			return
@@ -408,7 +379,8 @@ class UI(ui_cmd.UI):
 		os.chdir(newDir)
 		print(f"Changed current directory to: {newDir}")
 
-	def fs_cd(self, args: "List[str]"):
+	@staticmethod
+	def fs_cd(args: list[str]) -> None:
 		if len(args) != 1:
 			log.error("This command takes exactly one argument")
 			return
@@ -418,45 +390,112 @@ class UI(ui_cmd.UI):
 		os.chdir(newDir)
 		print(f"Changed current directory to: {newDir}")
 
-	def formatPromptMsg(self, level, msg, colon=":"):
-		indent = self.promptIndentStr * level
+	def formatPromptMsg(
+		self,
+		level: int,
+		msg: str,
+		colon: str = ":",
+	) -> tuple[str, bool]:
+		indent_ = self.promptIndentStr * level
 
 		if core.noColor:
-			return f"{indent} {msg}{colon} ", False
+			return f"{indent_} {msg}{colon} ", False
 
 		if self.promptIndentColor >= 0:
-			indent = f"\x1b[38;5;{self.promptIndentColor}m{indent}{endFormat}"
+			indent_ = f"\x1b[38;5;{self.promptIndentColor}m{indent_}{endFormat}"
 
 		if self.promptMsgColor >= 0:
 			msg = f"\x1b[38;5;{self.promptMsgColor}m{msg}{endFormat}"
 
-		return f"{indent} {msg}{colon} ", True
+		return f"{indent_} {msg}{colon} ", True
 
-	def prompt(self, level, msg, colon=":", **kwargs):
-		msg, colored = self.formatPromptMsg(level, msg, colon)
+	def formatPromptMsgStyleList(
+		self,
+		level: int,
+		msg: str,
+		colon: str = ":",
+	) -> StyleAndTextTuples:
+		indent_ = self.promptIndentStr * level
+		if core.noColor:
+			return [("", f"{indent_} {msg}{colon} ")]
+
+		indentStyle = ""
+		if self.promptIndentColor >= 0:
+			indentStyle = "fg:" + colors[self.promptIndentColor].hex
+
+		msgStyle = ""
+		if self.promptMsgColor >= 0:
+			msgStyle = "fg:" + colors[self.promptMsgColor].hex
+
+		return [
+			(indentStyle, f"{indent_} "),
+			(msgStyle, msg),
+			("", f"{colon} "),
+		]
+
+	def prompt(self, level: int, msg: str, colon: str = ":", **kwargs) -> str:
+		msg2, colored = self.formatPromptMsg(level, msg, colon)
 		if colored:
-			msg = ANSI(msg)
-		return prompt(msg, **kwargs)
+			msg2 = ANSI(msg2)
+		return prompt(msg2, **kwargs)
 
-	def checkbox_prompt(self, level, msg, colon=":", **kwargs):
-		# FIXME: colors are not working, they are being escaped
-		msg = f"{self.promptIndentStr * level} {msg}{colon} "
+	def checkbox_prompt(
+		self,
+		level: int,
+		msg: str,
+		default: bool,
+		colon: str = ":",
+	) -> bool:
+		"""Create a `PromptSession` object for the 'confirm' function."""
 		# msg, colored = self.formatPromptMsg(level, msg, colon)
-		return checkbox_prompt(msg, **kwargs)
+		bindings = KeyBindings()
 
-	def askFile(self, kind: str, histName: str, varName: str, reading: bool):
+		check = MiniCheckBoxPrompt(
+			formatted=self.formatPromptMsgStyleList(level, msg, colon=colon),
+			value=default,
+		)
+
+		@bindings.add(" ")
+		def space(_event: KeyPressEvent) -> None:
+			check.value = not check.value
+			# cursor_pos = check.formatMessage().find("[") + 1
+			# cur_cursor_pos = session.default_buffer.cursor_position
+			# print(f"{cur_cursor_pos=}, {cursor_pos=}")
+			# session.default_buffer.cursor_position = cursor_pos
+
+		@bindings.add(Keys.Any)
+		def _(_event: KeyPressEvent) -> None:
+			"""Disallow inserting other text."""
+
+		complete_message = check
+		session: PromptSession[bool] = PromptSession(
+			complete_message,
+			key_bindings=bindings,
+		)
+		session.prompt()
+		return check.value
+
+	def askFile(
+		self,
+		kind: str,
+		histName: str,
+		varName: str,
+		reading: bool,
+	) -> str:
 		from shlex import split as shlex_split
+
 		history = AbsolutePathHistory(join(histDir, histName))
 		auto_suggest = AutoSuggestFromHistory()
 		# Note: isdir and isfile funcs follow sym links, so no worry about links
 		completer = MyPathCompleter(
 			reading=reading,
-			fs_action_names=list(self._fsActions.keys()),
+			fs_action_names=list(self._fsActions),
 		)
 		default = getattr(self, varName)
 		while True:
 			filename = self.prompt(
-				1, kind,
+				1,
+				kind,
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -464,12 +503,17 @@ class UI(ui_cmd.UI):
 			)
 			if not filename:
 				continue
-			parts = shlex_split(filename)
+			try:
+				parts = shlex_split(filename)
+			except ValueError:
+				# file name can have single/double quote
+				setattr(self, varName, filename)
+				return filename
 			if parts[0] in self._fsActions:
 				actionFunc, usage = self._fsActions[parts[0]]
 				try:
 					actionFunc(parts[1:])
-				except Exception as e:
+				except Exception:
 					log.exception("")
 					if usage:
 						print("\n" + usage)
@@ -478,7 +522,7 @@ class UI(ui_cmd.UI):
 			return filename
 		raise ValueError(f"{kind} is not given")
 
-	def askInputFile(self):
+	def askInputFile(self) -> str:
 		return self.askFile(
 			"Input file",
 			"filename-input",
@@ -486,7 +530,7 @@ class UI(ui_cmd.UI):
 			True,
 		)
 
-	def askOutputFile(self):
+	def askOutputFile(self) -> str:
 		return self.askFile(
 			"Output file",
 			"filename-output",
@@ -494,7 +538,8 @@ class UI(ui_cmd.UI):
 			False,
 		)
 
-	def pluginByNameOrDesc(self, value: str) -> "Optional[PluginProp]":
+	@staticmethod
+	def pluginByNameOrDesc(value: str) -> PluginProp | None:
 		plugin = pluginByDesc.get(value)
 		if plugin:
 			return plugin
@@ -515,7 +560,8 @@ class UI(ui_cmd.UI):
 		)
 		while True:
 			value = self.prompt(
-				1, "Input format",
+				1,
+				"Input format",
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -539,7 +585,8 @@ class UI(ui_cmd.UI):
 		)
 		while True:
 			value = self.prompt(
-				1, "Output format",
+				1,
+				"Output format",
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -552,19 +599,20 @@ class UI(ui_cmd.UI):
 				return plugin.name
 		raise ValueError("output format is not given")
 
-	def finish(self):
+	def finish(self) -> None:
 		pass
 
 	# TODO: how to handle \r and \n in NewlineOption.values?
 
-	def getOptionValueSuggestValues(self, option: "option.Option"):
+	@staticmethod
+	def getOptionValueSuggestValues(option: Option) -> list[str] | None:
 		if option.values:
 			return [str(x) for x in option.values]
 		if option.typ == "bool":
 			return ["True", "False"]
 		return None
 
-	def getOptionValueCompleter(self, option: "option.Option"):
+	def getOptionValueCompleter(self, option: Option) -> WordCompleter | None:
 		values = self.getOptionValueSuggestValues(option)
 		if values:
 			return WordCompleter(
@@ -575,8 +623,8 @@ class UI(ui_cmd.UI):
 			)
 		return None
 
-	def askReadOptions(self):
-		plugin = Glossary.plugins[self._inputFormat]
+	# PLR0912 Too many branches (15 > 12)
+	def askReadOptions(self) -> None:  # noqa: PLR0912
 		options = Glossary.formatsReadOptions.get(self._inputFormat)
 		if options is None:
 			log.error(f"internal error: invalid format {self._inputFormat!r}")
@@ -585,7 +633,7 @@ class UI(ui_cmd.UI):
 		history = FileHistory(join(histDir, f"read-options-{self._inputFormat}"))
 		auto_suggest = AutoSuggestFromHistory()
 		completer = WordCompleter(
-			options.keys(),
+			list(options),
 			ignore_case=True,
 			match_middle=True,
 			sentence=True,
@@ -593,7 +641,8 @@ class UI(ui_cmd.UI):
 		while True:
 			try:
 				optName = self.prompt(
-					2, "ReadOption: Name (ENTER if done)",
+					2,
+					"ReadOption: Name (ENTER if done)",
 					history=history,
 					auto_suggest=auto_suggest,
 					completer=completer,
@@ -612,7 +661,8 @@ class UI(ui_cmd.UI):
 				if option.typ == "bool":
 					try:
 						valueNew = self.checkbox_prompt(
-							3, f"ReadOption: {optName}",
+							3,
+							f"ReadOption: {optName}",
 							default=default,
 						)
 					except (KeyboardInterrupt, EOFError):
@@ -622,7 +672,8 @@ class UI(ui_cmd.UI):
 					break
 				try:
 					value = self.prompt(
-						3, f"ReadOption: {optName}",
+						3,
+						f"ReadOption: {optName}",
 						colon=" =",
 						history=FileHistory(join(histDir, f"option-value-{optName}")),
 						auto_suggest=AutoSuggestFromHistory(),
@@ -631,25 +682,24 @@ class UI(ui_cmd.UI):
 					)
 				except (KeyboardInterrupt, EOFError):
 					break
-				if value == "":
+				if value == "" and option.typ != "str":  # noqa: PLC1901
 					if optName in self._readOptions:
 						print(f"Unset read-option {optName!r}")
 						del self._readOptions[optName]
-					# FIXME: set empty value?
 					break
 				valueNew, ok = option.evaluate(value)
 				if not ok or not option.validate(valueNew):
 					log.error(
 						f"Invalid read option value {optName}={value!r}"
-						f" for format {self._inputFormat}"
+						f" for format {self._inputFormat}",
 					)
 					continue
 				print(f"Set read-option: {optName} = {valueNew!r}")
 				self._readOptions[optName] = valueNew
 				break
 
-	def askWriteOptions(self):
-		plugin = Glossary.plugins[self._inputFormat]
+	# PLR0912 Too many branches (15 > 12)
+	def askWriteOptions(self) -> None:  # noqa: PLR0912
 		options = Glossary.formatsWriteOptions.get(self._outputFormat)
 		if options is None:
 			log.error(f"internal error: invalid format {self._outputFormat!r}")
@@ -658,7 +708,7 @@ class UI(ui_cmd.UI):
 		history = FileHistory(join(histDir, f"write-options-{self._outputFormat}"))
 		auto_suggest = AutoSuggestFromHistory()
 		completer = WordCompleter(
-			options.keys(),
+			list(options),
 			ignore_case=True,
 			match_middle=True,
 			sentence=True,
@@ -666,7 +716,8 @@ class UI(ui_cmd.UI):
 		while True:
 			try:
 				optName = self.prompt(
-					2, "WriteOption: Name (ENTER if done)",
+					2,
+					"WriteOption: Name (ENTER if done)",
 					history=history,
 					auto_suggest=auto_suggest,
 					completer=completer,
@@ -685,7 +736,8 @@ class UI(ui_cmd.UI):
 				if option.typ == "bool":
 					try:
 						valueNew = self.checkbox_prompt(
-							3, f"WriteOption: {optName}",
+							3,
+							f"WriteOption: {optName}",
 							default=default,
 						)
 					except (KeyboardInterrupt, EOFError):
@@ -695,7 +747,8 @@ class UI(ui_cmd.UI):
 					break
 				try:
 					value = self.prompt(
-						3, f"WriteOption: {optName}",
+						3,
+						f"WriteOption: {optName}",
 						colon=" =",
 						history=FileHistory(join(histDir, f"option-value-{optName}")),
 						auto_suggest=AutoSuggestFromHistory(),
@@ -704,38 +757,41 @@ class UI(ui_cmd.UI):
 					)
 				except (KeyboardInterrupt, EOFError):
 					break
-				if value == "":
+				if value == "" and option.typ != "str":  # noqa: PLC1901
 					if optName in self._writeOptions:
 						print(f"Unset write-option {optName!r}")
 						del self._writeOptions[optName]
-					# FIXME: set empty value?
 					break
 				valueNew, ok = option.evaluate(value)
 				if not ok or not option.validate(valueNew):
 					log.error(
 						f"Invalid write option value {optName}={value!r}"
-						f" for format {self._outputFormat}"
+						f" for format {self._outputFormat}",
 					)
 					continue
 				print(f"Set write-option: {optName} = {valueNew!r}")
 				self._writeOptions[optName] = valueNew
 				break
 
-	def resetReadOptions(self):
+	def resetReadOptions(self) -> None:
 		self._readOptions = {}
 
-	def resetWriteOptions(self):
+	def resetWriteOptions(self) -> None:
 		self._writeOptions = {}
 
-	def askConfigValue(self, configKey, option):
+	def askConfigValue(self, configKey: str, option: Option) -> str:
 		default = self.config.get(configKey, "")
 		if option.typ == "bool":
-			return str(self.checkbox_prompt(
-				3, f"Config: {configKey}",
-				default=bool(default),
-			))
+			return str(
+				self.checkbox_prompt(
+					3,
+					f"Config: {configKey}",
+					default=bool(default),
+				),
+			)
 		return self.prompt(
-			3, f"Config: {configKey}",
+			3,
+			f"Config: {configKey}",
 			colon=" =",
 			history=FileHistory(join(histDir, f"config-value-{configKey}")),
 			auto_suggest=AutoSuggestFromHistory(),
@@ -743,9 +799,9 @@ class UI(ui_cmd.UI):
 			completer=self.getOptionValueCompleter(option),
 		)
 
-	def askConfig(self):
-		configKeys = list(sorted(self.configDefDict.keys()))
-		history = FileHistory(join(histDir, f"config-key"))
+	def askConfig(self) -> None:
+		configKeys = sorted(self.configDefDict)
+		history = FileHistory(join(histDir, "config-key"))
 		auto_suggest = AutoSuggestFromHistory()
 		completer = WordCompleter(
 			configKeys,
@@ -757,7 +813,8 @@ class UI(ui_cmd.UI):
 		while True:
 			try:
 				configKey = self.prompt(
-					2, "Config: Key (ENTER if done)",
+					2,
+					"Config: Key (ENTER if done)",
 					history=history,
 					auto_suggest=auto_suggest,
 					completer=completer,
@@ -772,16 +829,15 @@ class UI(ui_cmd.UI):
 					value = self.askConfigValue(configKey, option)
 				except (KeyboardInterrupt, EOFError):
 					break
-				if value == "":
+				if value == "" and option.typ != "str":  # noqa: PLC1901
 					if configKey in self.config:
 						print(f"Unset config {configKey!r}")
 						del self.config[configKey]
-					# FIXME: set empty value?
 					break
 				valueNew, ok = option.evaluate(value)
 				if not ok or not option.validate(valueNew):
 					log.error(
-						f"Invalid config value {configKey}={value!r}"
+						f"Invalid config value {configKey}={value!r}",
 					)
 					continue
 				print(f"Set config: {configKey} = {valueNew!r}")
@@ -789,39 +845,39 @@ class UI(ui_cmd.UI):
 				self.config[configKey] = valueNew
 				break
 
-	def showOptions(self):
+	def showOptions(self) -> None:
 		print(f"readOptions = {self._readOptions}")
 		print(f"writeOptions = {self._writeOptions}")
 		print(f"convertOptions = {self._convertOptions}")
 		print(f"config = {self.config}")
 		print()
 
-	def setIndirect(self):
+	def setIndirect(self) -> None:
 		self._convertOptions["direct"] = False
 		self._convertOptions["sqlite"] = None
 		print("Switched to indirect mode")
 
-	def setSQLite(self):
+	def setSQLite(self) -> None:
 		self._convertOptions["direct"] = None
 		self._convertOptions["sqlite"] = True
 		print("Switched to SQLite mode")
 
-	def setNoProgressbar(self):
-		self._convertOptions["progressbar"] = False
+	def setNoProgressbar(self) -> None:
+		self._glossarySetAttrs["progressbar"] = False
 		print("Disabled progress bar")
 
-	def setSort(self):
-		from pyglossary.entry import Entry
+	def setSort(self) -> None:
 		try:
 			value = self.checkbox_prompt(
-				2, f"Enable Sort",
+				2,
+				"Enable Sort",
 				default=self._convertOptions.get("sort", False),
 			)
 		except (KeyboardInterrupt, EOFError):
 			return
 		self._convertOptions["sort"] = value
 
-	def setSortKey(self):
+	def setSortKey(self) -> None:
 		completer = WordCompleter(
 			[_sk.name for _sk in namedSortKeyList],
 			ignore_case=False,
@@ -830,8 +886,9 @@ class UI(ui_cmd.UI):
 		)
 		default = self._convertOptions.get("sortKeyName", "")
 		sortKeyName = self.prompt(
-			2, "SortKey",
-			history=FileHistory(join(histDir, f"sort-key")),
+			2,
+			"SortKey",
+			history=FileHistory(join(histDir, "sort-key")),
 			auto_suggest=AutoSuggestFromHistory(),
 			default=default,
 			completer=completer,
@@ -841,8 +898,8 @@ class UI(ui_cmd.UI):
 				del self._convertOptions["sortKeyName"]
 			return
 
-		if sortKeyName not in namedSortKeyByName:
-			log.error(f"invalid sortKeyName = {sortKeyName!r}")
+		if not lookupSortKey(sortKeyName):
+			log.error(f"invalid {sortKeyName = }")
 			return
 
 		self._convertOptions["sortKeyName"] = sortKeyName
@@ -850,18 +907,19 @@ class UI(ui_cmd.UI):
 		if not self._convertOptions.get("sort"):
 			self.setSort()
 
-	def askFinalAction(self) -> "Optional[str]":
+	def askFinalAction(self) -> str | None:
 		history = FileHistory(join(histDir, "action"))
 		auto_suggest = AutoSuggestFromHistory()
 		completer = WordCompleter(
-			list(self._finalActions.keys()),
+			list(self._finalActions),
 			ignore_case=False,
 			match_middle=True,
 			sentence=True,
 		)
 		while True:
 			action = self.prompt(
-				1, "Select action (ENTER to convert)",
+				1,
+				"Select action (ENTER to convert)",
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -873,13 +931,13 @@ class UI(ui_cmd.UI):
 				continue
 			return action
 
-	def askFinalOptions(self) -> "Union[bool, Literal[back]]":
+	def askFinalOptions(self) -> bool | str:
 		while True:
 			try:
 				action = self.askFinalAction()
 			except (KeyboardInterrupt, EOFError):
 				return False
-			except Exception as e:
+			except Exception:
 				log.exception("")
 				return False
 			if action == back:
@@ -893,67 +951,71 @@ class UI(ui_cmd.UI):
 
 		return True  # convert
 
-	def getRunKeywordArgs(self) -> "Dict":
-		return dict(
-			inputFilename=self._inputFilename,
-			outputFilename=self._outputFilename,
-			inputFormat=self._inputFormat,
-			outputFormat=self._outputFormat,
-			config=self.config,
-			readOptions=self._readOptions,
-			writeOptions=self._writeOptions,
-			convertOptions=self._convertOptions,
-			glossarySetAttrs=self._glossarySetAttrs,
-		)
+	def getRunKeywordArgs(self) -> dict:
+		return {
+			"inputFilename": self._inputFilename,
+			"outputFilename": self._outputFilename,
+			"inputFormat": self._inputFormat,
+			"outputFormat": self._outputFormat,
+			"config": self.config,
+			"readOptions": self._readOptions,
+			"writeOptions": self._writeOptions,
+			"convertOptions": self._convertOptions,
+			"glossarySetAttrs": self._glossarySetAttrs,
+		}
 
-	def checkInputFormat(self, forceAsk: bool = False):
+	def checkInputFormat(self, forceAsk: bool = False) -> None:
 		if not forceAsk:
-			inputArgs = Glossary.detectInputFormat(self._inputFilename, quiet=True)
-			if inputArgs:
-				inputFormat = inputArgs[1]
+			try:
+				inputArgs = Glossary.detectInputFormat(self._inputFilename)
+			except Error:
+				pass
+			else:
+				inputFormat = inputArgs.formatName
 				self._inputFormat = inputFormat
 				return
 		self._inputFormat = self.askInputFormat()
 
-	def checkOutputFormat(self, forceAsk: bool = False):
+	def checkOutputFormat(self, forceAsk: bool = False) -> None:
 		if not forceAsk:
-			outputArgs = Glossary.detectOutputFormat(
-				filename=self._outputFilename,
-				inputFilename=self._inputFilename,
-				quiet=True,
-			)
-			if outputArgs:
-				self._outputFormat = outputArgs[1]
+			try:
+				outputArgs = Glossary.detectOutputFormat(
+					filename=self._outputFilename,
+					inputFilename=self._inputFilename,
+				)
+			except Error:
+				pass
+			else:
+				self._outputFormat = outputArgs.formatName
 				return
 		self._outputFormat = self.askOutputFormat()
 
-	def askFormats(self):
+	def askFormats(self) -> None:
 		self.checkInputFormat(forceAsk=True)
 		self.checkOutputFormat(forceAsk=True)
 
-	def askInputOutputAgain(self):
+	def askInputOutputAgain(self) -> None:
 		self.askInputFile()
 		self.checkInputFormat(forceAsk=True)
 		self.askOutputFile()
 		self.checkOutputFormat(forceAsk=True)
 
-	def printNonInteractiveCommand(self):
-		from shlex import quote
+	def printNonInteractiveCommand(self) -> None:  # noqa: PLR0912
 		cmd = [
 			ui_cmd.COMMAND,
-			quote(self._inputFilename),
-			quote(self._outputFilename),
-			quote(f"--read-format={self._inputFormat}"),
-			quote(f"--write-format={self._outputFormat}"),
+			self._inputFilename,
+			self._outputFilename,
+			f"--read-format={self._inputFormat}",
+			f"--write-format={self._outputFormat}",
 		]
 
 		if self._readOptions:
 			optionsJson = json.dumps(self._readOptions, ensure_ascii=True)
-			cmd.append(quote(f"--json-read-options={optionsJson}"))
+			cmd += ["--json-read-options", optionsJson]
 
 		if self._writeOptions:
 			optionsJson = json.dumps(self._writeOptions, ensure_ascii=True)
-			cmd.append(quote(f"--json-write-options={optionsJson}"))
+			cmd += ["--json-write-options", optionsJson]
 
 		if self.config:
 			for key, value in self.config.items():
@@ -968,13 +1030,13 @@ class UI(ui_cmd.UI):
 					log.error(f"config key {key} has no command line flag")
 				flag = option.customFlag
 				if not flag:
-					flag = key.replace('_', '-')
+					flag = key.replace("_", "-")
 				if option.typ == "bool":
 					if not value:
 						flag = f"no-{flag}"
 					cmd.append(f"--{flag}")
 				else:
-					cmd.append(quote(f"--{flag}={value}"))
+					cmd.append(f"--{flag}={value}")
 
 		if self._convertOptions:
 			if "infoOverride" in self._convertOptions:
@@ -982,7 +1044,7 @@ class UI(ui_cmd.UI):
 				for key, value in infoOverride.items():
 					flag = infoOverrideFlags.get(key)
 					if not flag:
-						log.error(f"unknow key {key} in infoOverride")
+						log.error(f"unknown key {key} in infoOverride")
 						continue
 					cmd.append(f"--{flag}={value}")
 
@@ -994,7 +1056,7 @@ class UI(ui_cmd.UI):
 				if value is None:
 					continue
 				if key not in convertOptionsFlags:
-					log.error(f"unknow key {key} in convertOptions")
+					log.error(f"unknown key {key} in convertOptions")
 					continue
 				ftup = convertOptionsFlags[key]
 				if ftup is None:
@@ -1007,83 +1069,48 @@ class UI(ui_cmd.UI):
 					flag = ftup[0]
 					cmd.append(f"--{flag}={value}")
 
+		if (
+			"progressbar" in self._glossarySetAttrs
+			and not self._glossarySetAttrs["progressbar"]
+		):
+			cmd.append("--no-progress-bar")
+
 		print()
 		print(
-			"If you want to repeat this conversion later, "
-			"you can use this command:"
+			"If you want to repeat this conversion later, you can use this command:",
 		)
+		# shlex.join is added in Python 3.8
 		print(shlex.join(cmd))
 
-	def setConfigAttrs(self):
+	def setConfigAttrs(self) -> None:
 		config = self.config
 		self.promptIndentStr = config.get("cmdi.prompt.indent.str", ">")
 		self.promptIndentColor = config.get("cmdi.prompt.indent.color", 2)
 		self.promptMsgColor = config.get("cmdi.prompt.msg.color", -1)
 		self.msgColor = config.get("cmdi.msg.color", -1)
 
-	def run(
-		self,
-		inputFilename: str = "",
-		outputFilename: str = "",
-		inputFormat: str = "",
-		outputFormat: str = "",
-		reverse: bool = False,
-		config: "Optional[Dict]" = None,
-		readOptions: "Optional[Dict]" = None,
-		writeOptions: "Optional[Dict]" = None,
-		convertOptions: "Optional[Dict]" = None,
-		glossarySetAttrs: "Optional[Dict]" = None,
-	):
-		if config is None:
-			config = {}
-		if readOptions is None:
-			readOptions = {}
-		if writeOptions is None:
-			writeOptions = {}
-		if convertOptions is None:
-			convertOptions = {}
-		if glossarySetAttrs is None:
-			glossarySetAttrs = {}
-
-		self._inputFilename = inputFilename
-		self._outputFilename = outputFilename
-		self._inputFormat = inputFormat
-		self._outputFormat = outputFormat
-		self._readOptions = readOptions
-		self._writeOptions = writeOptions
-		self._convertOptions = convertOptions
-		self._glossarySetAttrs = glossarySetAttrs
-
-
-		self.loadConfig()
-		self.savedConfig = dict(self.config)
-		self.config = config
-
-		del inputFilename, outputFilename, inputFormat, outputFormat
-		del config, readOptions, writeOptions, convertOptions
-
-		self.setConfigAttrs()
-
-		if not self._inputFilename:
+	# PLR0912 Too many branches (19 > 12)
+	def main(self, again: bool = False) -> None:  # noqa: PLR0912
+		if again or not self._inputFilename:
 			try:
 				self.askInputFile()
 			except (KeyboardInterrupt, EOFError):
-				return
-		if not self._inputFormat:
+				return None
+		if again or not self._inputFormat:
 			try:
 				self.checkInputFormat()
 			except (KeyboardInterrupt, EOFError):
-				return
-		if not self._outputFilename:
+				return None
+		if again or not self._outputFilename:
 			try:
 				self.askOutputFile()
 			except (KeyboardInterrupt, EOFError):
-				return
-		if not self._outputFormat:
+				return None
+		if again or not self._outputFormat:
 			try:
 				self.checkOutputFormat()
 			except (KeyboardInterrupt, EOFError):
-				return
+				return None
 
 		while True:
 			status = self.askFinalOptions()
@@ -1091,15 +1118,70 @@ class UI(ui_cmd.UI):
 				self.askInputOutputAgain()
 				continue
 			if not status:
-				return
+				return None
 			try:
 				succeed = ui_cmd.UI.run(self, **self.getRunKeywordArgs())
-			except Exception as e:
+			except Exception:
 				log.exception("")
 			else:
 				self.printNonInteractiveCommand()
 				if succeed:
-					if self.config != self.savedConfig and confirm("Save Config?"):
-						self.saveConfig()
 					return succeed
 			print("Press Control + C to exit")
+
+	def run(  # noqa: PLR0913
+		self,
+		inputFilename: str = "",
+		outputFilename: str = "",
+		inputFormat: str = "",
+		outputFormat: str = "",
+		reverse: bool = False,
+		config: dict[str, Any] | None = None,
+		readOptions: dict[str, Any] | None = None,
+		writeOptions: dict[str, Any] | None = None,
+		convertOptions: dict[str, Any] | None = None,
+		glossarySetAttrs: dict[str, Any] | None = None,
+	) -> bool:
+		if reverse:
+			raise NotImplementedError("Reverse is not implemented in this UI")
+
+		self._inputFilename = inputFilename
+		self._outputFilename = outputFilename
+		self._inputFormat = inputFormat
+		self._outputFormat = outputFormat
+		self._readOptions = readOptions or {}
+		self._writeOptions = writeOptions or {}
+		self._convertOptions = convertOptions or {}
+		self._glossarySetAttrs = glossarySetAttrs or {}
+
+		if not self._progressbar:
+			self._glossarySetAttrs["progressbar"] = False
+
+		self.loadConfig()
+		self.savedConfig = self.config.copy()
+		self.config = config or {}
+
+		del inputFilename, outputFilename, inputFormat, outputFormat
+		del config, readOptions, writeOptions, convertOptions
+
+		self.setConfigAttrs()
+
+		self.main()
+
+		try:
+			while (
+				self.prompt(
+					level=1,
+					msg="Press enter to exit, 'a' to convert again",
+					default="",
+				)
+				== "a"
+			):
+				self.main(again=True)
+		except KeyboardInterrupt:
+			pass
+
+		if self.config != self.savedConfig and confirm("Save Config?"):
+			self.saveConfig()
+
+		return True
