@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# mypy: ignore-errors
 # ui_qk.py
 #
 # Copyright © 2010-2019 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
@@ -16,19 +17,20 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
 # GNU General Public License for more details.
 
-
-from pyglossary.glossary import *
-from .base import *
+from __future__ import annotations
 from os.path import join
 
 from PyQt4 import QtGui as qt
-from PyQt4 import QtCore as qc
+
+from pyglossary.glossary_v2 import *
+
+from .base import *
 
 noneItem = 'Not Selected'
 
 
 class UI(qt.QWidget, UIBase):
-	def __init__(self):
+	def __init__(self) -> None:
 		qt.QWidget.__init__(self)
 		UIBase.__init__(self)
 		self.setWindowTitle('PyGlossary (Qt)')
