@@ -1,3 +1,5 @@
+#!/usr/bin/env python3
+
 import sys
 from pprint import pformat
 
@@ -6,8 +8,6 @@ from pyglossary.glossary import Glossary
 glos = Glossary()
 glos.read(sys.argv[1])
 for entry in glos:
-	print('Words: ' + pformat(entry.l_word))
-	print('Definitions: ' + pformat(entry.defis))
-	print('-------------------------')
-
-
+	print("Words: " + pformat(entry.l_word))
+	print("Definitions: " + pformat(entry.defis))
+	print("-------------------------")
