@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 
 import sys
-from os.path import join, dirname, abspath
+from os.path import abspath, dirname
 
 rootDir = dirname(dirname(abspath(__file__)))
 sys.path.insert(0, rootDir)
@@ -10,10 +10,7 @@ from pyglossary import Glossary
 
 
 def hasBar(entry):
-	for word in entry.l_word:
-		if "|" in word:
-			return True
-	return False
+	return any("|" in word for word in entry.l_word)
 
 
 Glossary.init(
@@ -21,7 +18,7 @@ Glossary.init(
 )
 
 for direct in (True, False):
-	print(f"\n-------- direct={direct}")
+	print(f"\n-------- {direct=}")
 
 	glos = Glossary()
 	glos.config = {
@@ -36,4 +33,4 @@ for direct in (True, False):
 			print(f"+++ {entry.l_word!r} -> {entry.defi[:60]}")
 			continue
 
-		#print(f"--- {entry.l_word!r} -> {entry.defi[:60]}")
+		# print(f"--- {entry.l_word!r} -> {entry.defi[:60]}")
