@@ -1,18 +1,16 @@
 #!/usr/bin/python3
 
 import sys
-import json
-from os.path import join, dirname, abspath
+from os.path import abspath, dirname, join
 from pathlib import Path
-from collections import OrderedDict
+
 import toml
-from pprint import pprint
 
 rootDir = dirname(dirname(abspath(__file__)))
 sys.path.insert(0, rootDir)
 
-from pyglossary.glossary import Glossary
 from pyglossary.core import userPluginsDir
+from pyglossary.glossary import Glossary
 
 Glossary.init(
 	# usePluginsJson=False,
@@ -20,9 +18,7 @@ Glossary.init(
 
 userPluginsDirPath = Path(userPluginsDir)
 plugins = [
-	p
-	for p in Glossary.plugins.values()
-	if userPluginsDirPath not in p.path.parents
+	p for p in Glossary.plugins.values() if userPluginsDirPath not in p.path.parents
 ]
 
 toolsDir = join(rootDir, "plugins-meta", "tools")
@@ -31,7 +27,7 @@ for p in plugins:
 	module = p.module
 	optionsProp = p.optionsProp
 
-	tools = OrderedDict()
+	tools = {}
 	for tool in getattr(p.module, "tools", []):
 		tools[tool.pop("name")] = tool
 
@@ -40,7 +36,5 @@ for p in plugins:
 
 	# pprint(tools)
 
-	with open(join(toolsDir, f"{p.lname}.toml"), mode="w") as _file:
+	with open(join(toolsDir, f"{p.lname}.toml"), mode="w", encoding="utf-8") as _file:
 		toml.dump(tools, _file)
-
-
