@@ -1,20 +1,16 @@
-import sys
-from os.path import dirname, abspath
 import unittest
 
-rootDir = dirname(dirname(abspath(__file__)))
-sys.path.insert(0, rootDir)
-
-from tests.glossary_test import TestGlossaryBase
-from pyglossary.glossary import Glossary
+from glossary_v2_test import TestGlossaryBase
 
 
 class TestGlossaryDictionaryForMIDs(TestGlossaryBase):
 	def __init__(self, *args, **kwargs):
 		TestGlossaryBase.__init__(self, *args, **kwargs)
-		self.dataFileCRC32.update({
-			"100-en-fa.mids.zip": "32d1185f",
-		})
+		self.dataFileCRC32.update(
+			{
+				"100-en-fa.mids.zip": "32d1185f",
+			},
+		)
 
 	def convert_txt_mids(self, fname, sha1sumDict, **convertArgs):
 		outputFname = f"{fname}-2.mids.zip"
@@ -23,17 +19,16 @@ class TestGlossaryDictionaryForMIDs(TestGlossaryBase):
 		self.convert(
 			f"{fname}.txt",
 			outputFname,
-			**convertArgs
+			**convertArgs,
 		)
 		self.checkZipFileSha1sum(outputFpath, sha1sumDict)
 
 	def test_convert_txt_mids_1(self):
 		sha1sumDict = {
-			"DictionaryForMIDs.properties": \
-				"4260a87d6cdd55622dcfe395880bc913f96102b8",
-			"directory1.csv": "70b0e683f2f4c9246500974a87467a3210d099c2",
-			"index1.csv": "5033902993e44257fce29df8443481958a101602",
-			"searchlist.csv": "d6f144dd001c7df79edb459fc9530515a747224d",
+			"DictionaryForMIDs.properties": "4260a87d6cdd55622dcfe395880bc913f96102b8",
+			"directory1.csv": "1f1ab12b107608a1513254fff3c323bbcdfbd5cf",
+			"index1.csv": "494268da410c520e56142b47610f6bbcfd53c79f",
+			"searchlist.csv": "4f4513d1550436e867e1a79dbd073a7e5bb38e32",
 		}
 		self.convert_txt_mids("100-en-fa", sha1sumDict)
 
