@@ -1,16 +1,19 @@
-import sys
-from os.path import dirname, abspath
-import unittest
 import gzip
+import os
+import unittest
 
-rootDir = dirname(dirname(abspath(__file__)))
-sys.path.insert(0, rootDir)
-
-from tests.glossary_test import TestGlossaryBase
-from pyglossary.glossary import Glossary
+from glossary_v2_test import TestGlossaryBase
 
 
 class TestGlossaryKobo(TestGlossaryBase):
+	def setUp(self):
+		if os.getenv("SKIP_MISSING"):
+			try:
+				import marisa_trie  # noqa: F401
+			except ImportError:
+				self.skipTest("skipping module due to missing dependency: marisa_trie")
+		TestGlossaryBase.setUp(self)
+
 	def __init__(self, *args, **kwargs):
 		TestGlossaryBase.__init__(self, *args, **kwargs)
 		# self.dataFileCRC32.update({})
@@ -22,17 +25,15 @@ class TestGlossaryKobo(TestGlossaryBase):
 		self.convert(
 			f"{fname}.txt",
 			outputFname,
-			**convertArgs
+			**convertArgs,
 		)
 		dataReplaceFuncs = {
-			_zfname: gzip.decompress
-			for _zfname in sha1sumDict
-			if _zfname != "words"
+			_zfname: gzip.decompress for _zfname in sha1sumDict if _zfname != "words"
 		}
 		self.checkZipFileSha1sum(
 			outputFpath,
 			sha1sumDict=sha1sumDict,
-			dataReplaceFuncs=dataReplaceFuncs
+			dataReplaceFuncs=dataReplaceFuncs,
 		)
 
 	def test_convert_txt_kobo_1(self):
@@ -40,9 +41,10 @@ class TestGlossaryKobo(TestGlossaryBase):
 			"11.html": "39f0f46560da7398ab0d3b19cc1c2387ecd201dd",
 			"aa.html": "df9460450e8b46e913c57bf39dcc799ffdc2fb33",
 			"ab.html": "be4271a8508dbb499bafd439810af621a7b3474f",
-			"words": "d0f74e854f090fbaa8211bcfd162ad99ec4da0a3",  
+			"words": "d0f74e854f090fbaa8211bcfd162ad99ec4da0a3",
 		}
 		self.convert_txt_kobo("100-en-fa", sha1sumDict)
 
+
 if __name__ == "__main__":
 	unittest.main()
